/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/contact-ext', [
    'io.ox/core/extensions',
    'io.ox/core/extPatterns/links',
    'io.ox/core/extPatterns/actions',
    'io.ox/contacts/model',
    'io.ox/core/capabilities',
    'io.ox/core/notifications',
    'com.voiceworks/ox-messenger/core/session',
    'com.voiceworks/ox-messenger/roster/main',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/history/main',
    'com.voiceworks/ox-messenger/history/views/small',
    'com.voiceworks/ox-messenger/history/models/historycollection',
    'gettext!com.voiceworks/ox-messenger'
], function (ext, links, actions, contactModel, cap, notifications, Session, RosterController, Aggregator, History, SmallHistoryView, HistoryCollection, gt) {
    'use strict';

    var Action = links.Action,
        ActionLink = links.ActionLink;

    var ContactHistoryView = SmallHistoryView.extend({
        showFullHistory: function (e) {

            window.avoidShowingLastConversation = true;

            $('.window-body').trigger('click'); // close halo
            History.show();
        },

        showEmptyMessage: function () {
            return $('<div class="alert alert-info">').text(gt('No call history with this user'));
        },

        pstnAction: function (phoneNumber) {
            Aggregator.trigger('phone:toggle', { phoneNumber: phoneNumber });
        },

        setJid: function (jid) {
            this.jid = jid;
            this.historyUpdated();
            this.render();
        },

        getHistory: function () {
            if (!this.jid) {
                return new HistoryCollection();
            }

            var user = RosterController.getUserById(this.jid);
            return Session.user.callHistory.getContactHistory(user, 5);
        }
    });

    ext.point('io.ox/contacts/detail/content').extend({
        id: 'messenger-call-history',
        index: 1000,
        draw: function (baton) {
            var historyView = new ContactHistoryView();
            historyView.setJid(baton.data.email1);

            this.append(
                $('<fieldset class="block">').append(
                    $('<legend>').text(gt('Call History')),
                    historyView.render({halo: true}).$el
                )
            );
        }
    });

    new Action('io.ox/contacts/head/call-audio', {
        'capabilities': 'messenger-call',
        requires: function (e) {
            if (e.baton.data instanceof Array === true || !RosterController.getUserById(e.baton.data.email1) || Session.user.canCallWith('audio', RosterController.getUserById(e.baton.data.email1), false) !== true) {
                return false;
            }

            return true;
        },
        action: function (baton) {
            $('.window-body').trigger('click'); // close halo

            if (Session.user.call.isActive()) {
                notifications.yell('error', gt('You are already in a call.'));
                return;
            }

            Aggregator.trigger('conversation:show', { id: baton.data.email1, type: 'audio' });
        }
    });

    new Action('io.ox/contacts/head/call-video', {
        'capabilities': 'messenger-call',
        requires: function (e) {
            if (e.baton.data instanceof Array === true ||
                !RosterController.getUserById(e.baton.data.email1) ||
                Session.user.canCallWith('video', RosterController.getUserById(e.baton.data.email1), false) !== true) {
                return false;
            }
            return true;
        },
        action: function (baton) {
            $('.window-body').trigger('click'); // close halo

            if (Session.user.call.isActive()) {
                notifications.yell('error', gt('You are already in a call.'));
                return;
            }

            Aggregator.trigger('conversation:show', { id: baton.data.email1, type: 'video' });
        }
    });

    new Action('io.ox/contacts/head/chat', {
        requires: function (e) {
            if (e.baton.data instanceof Array === true ||
                !RosterController.getUserById(e.baton.data.email1) ||
                Session.user.canChatWith(RosterController.getUserById(e.baton.data.email1)) !== true) {
                return false;
            }
            return true;
        },
        action: function (baton) {
            $('.window-body').trigger('click'); // close halo

            Aggregator.trigger('conversation:show', { id: baton.data.email1, type: 'chat' });
        }
    });

    new Action('io.ox/contacts/head/group-chat', {
        'capabilities': 'messenger-group',
        requires: function (e) {
            /*if (e.baton.data instanceof Array === false || !(RosterController.getUserById(e.baton.data.email1))) {
                return false;
            }

            return true;*/
        },
        action: function (baton) {

            var roomName = '';

            $.each(baton.data, function (index, data) {
                roomName += data.display_name + ', ';
            });

            roomName = roomName.substring(0, roomName.length - 2);

            /*convModel.set('id', convModel.get('roomName').toLowerCase() + '@' + convModel.get('roomHost').toLowerCase());
            convModel.set('timestamp', new Date().getTime());
            convModel.set('lastmsg', 'Group started');
            convModel.set('myRoom', true);

            require(['com.voiceworks/ox-messenger/main'], function (m) {
                m.getApp().launch();
                m.getApp().getConversation(baton.data.email1, "chat", true, m.getApp().showConversation);
            });*/
        }
    });

    ext.point('io.ox/contacts/classic-toolbar/links').extend(new links.Link({
        id: 'Messenger',
        index: 1100,
        prio: 'hi',
        mobile: 'hi',
        label: gt('Messenger'),
        drawDisabled: true,
        ref: 'io.ox/contacts/dropdown/messenger',
        customize: function (baton) {

            this.append('<i class="fa fa-caret-down">');

            this.after(
                links.DropdownLinks({ ref: 'io.ox/contacts/links/toolbar/messenger', wrap: false }, baton)
            );

            this.addClass('dropdown-toggle').attr({
                'aria-haspopup': 'true',
                'data-toggle': 'dropdown',
                'role': 'button'
            });

            this.parent().addClass('dropdown');
        }
    }));

    ext.point('io.ox/contacts/links/inline').extend(new links.Link({
        id: 'Messenger',
        index: 1100,
        prio: 'hi',
        mobile: 'hi',
        label: gt('Messenger'),
        ref: 'io.ox/contacts/dropdown/messenger',
        customize: function (baton) {

            this.append('<i class="fa fa-caret-down">');

            this.after(
                links.DropdownLinks({ ref: 'io.ox/contacts/links/toolbar/messenger', wrap: false }, baton)
            );

            this.addClass('dropdown-toggle').attr({
                'aria-haspopup': 'true',
                'data-toggle': 'dropdown',
                'role': 'button'
            });

            this.parent().addClass('dropdown');
        }
    }));

    new actions.Action('io.ox/contacts/dropdown/messenger', {
        requires: function (e) {
            if (e.baton.data instanceof Object === true) {
                var allAvailable = true;

                $.each(e.baton.data, function (id, data) {
                    if (id === 'email1' && checkUser(data) === false) {
                        allAvailable = false;
                    }
                });

                return allAvailable;
            } else {
                var test = checkUser(e.baton.data.email1);
                return test;
            }
            return false;
        },
        action: $.noop
    });

    new ActionLink('io.ox/contacts/links/toolbar/messenger', {
        index: 100,
        id: 'chat',
        label: gt('Chat'),
        ref: 'io.ox/contacts/head/chat'
    });

    new ActionLink('io.ox/contacts/links/toolbar/messenger', {
        index: 300,
        id: 'group-chat',
        label: gt('Start groupchat'),
        ref: 'io.ox/contacts/head/group-chat'
    });

    new ActionLink('io.ox/contacts/links/toolbar/messenger', {
        index: 300,
        id: 'call-audio',
        label: gt('Call (audio)'),
        ref: 'io.ox/contacts/head/call-audio'
    });

    new ActionLink('io.ox/contacts/links/toolbar/messenger', {
        index: 400,
        id: 'call-video',
        label: gt('Call (video)'),
        ref: 'io.ox/contacts/head/call-video'
    });

    new ActionLink('io.ox/contacts/links/toolbar/messenger', {
        index: 500,
        id: 'invite-buddy',
        label: gt('Add to buddylist'),
        ref: 'io.ox/contacts/head/invite-buddy'
    });

    var checkUser = function (email) {
        if (email === Session.user.id) {
            return false;
        }

        if (!RosterController.getUserById(email)) {
            return false;
        }

        if (Session.user.canChatWith(RosterController.getUserById(email)) !== true) {
            return false;
        }

        if (RosterController.getUserById(email).status === 'offline') {
            return false;
        }

        return true;
    };

    function block() {

        function removeAndSplice() {
            rows.splice(-1, 1);
            return args.slice(-1)[0];
        }

        var args = _(arguments).toArray(),
            rows = _(args.slice(1)).compact(),
            noDl = _.isBoolean(args.slice(-1)[0]) ? removeAndSplice() : false,
            block = $('<fieldset class="block">'),
            dl = $('<dl class="dl-horizontal">'),
            self = noDl ? block : dl.appendTo(block);

        // if block empty
        if (rows.length < 1) {
            return $();
        }

        _.each(rows, function (el) {
            self.append(el);
        });

        return block.prepend(
            $('<legend>').text(args[0])
        );
    }

    function simple(data, id, label) {
        var value = $.trim(data[id]);
        if (!value){return null;}
        return function () {
            $(this).append(
                $('<dt>').text(label || contactModel.fields[id]),
                $('<dd>').text(value)
            );
        };
    }

    ext.point('io.ox/contacts/detail/content').extend({
        id: 'phone',
        index: 500,
        draw: function (baton) {
            var data = baton.data;

            this.append(
                block(gt('Phone numbers'),
                    phone(data, 'cellular_telephone1'),
                    phone(data, 'cellular_telephone2'),
                    phone(data, 'telephone_business1'),
                    phone(data, 'telephone_business2'),
                    phone(data, 'telephone_home1'),
                    phone(data, 'telephone_home2'),
                    phone(data, 'telephone_other'),
                    simple(data, 'fax_business'),
                    simple(data, 'fax_home'),
                    simple(data, 'fax_other')
                )
                .attr('data-block', 'phone')
            );
        }
    });

    function phone(data, id, label) {
        var number = $.trim(data[id]);

        if (!number) {
            return null;
        }

        return function () {
            if (cap.has('messenger-phone')) {
                $(this).append(
                    $('<dt>').text(label || contactModel.fields[id]),
                    $('<dd>').append(
                        $('<a>', { href: number }).text(number).click(function (e) {
                            e.preventDefault();

                            // Show PSTN sidebar and prefill number
                            Aggregator.trigger('phone:toggle', { phoneNumber: number });
                        })
                    )
                );
            } else {
                $(this).append(
                    $('<dt>').text(label || contactModel.fields[id]),
                    $('<dd>').append(
                        $('<a>', {
                    href: _.device('smartphone') ? 'tel:' + number : 'callto:' + number
                }).text(number)
                    )
                );
            }
        };
    }
});
