/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/conversation/models/conversationcollection', [
    'com.voiceworks/ox-messenger/conversation/models/conversation'
], function (ConversationModel) {
    'use strict';

    /**
     * Conversation Collection
     */
    var ConversationCollection = Backbone.Collection.extend({

        model: ConversationModel,

        currentConversationId: null,

        comparator: function (modelA, modelB) {
            if (modelA.get(this.sortMethod) === modelB.get(this.sortMethod)) {
                return 0;
            }

            if (modelA.get(this.sortMethod).toString().toLowerCase() > modelB.get(this.sortMethod).toString().toLowerCase()) {
                return this.sortDirection === 'asc' ? 1 : -1;
            } else {
                return this.sortDirection === 'asc' ? -1 : 1;
            }
        },

        fetchMucRooms: function () {
            var rooms = this.filter(function (conversation) {
                return conversation.get('roomId');
            });
            return rooms;
        },

        updateAppUnreadBadge: function () {
            var unreads = this.pluck('unreadMessages'),
                unreadCount = 0,
                appActive = ox.ui.App.getCurrentApp().get('name') === 'com.voiceworks/ox-messenger';

            _.each(unreads, function (count) {
                unreadCount += (count === undefined) ? 0 : count;
            });

            if (unreadCount < 1 || appActive) {
                $('.messenger--badge-unread').hide();
                return;
            }

            if (!$('.messenger--badge-unread').length) {
                $('.launcher[data-app-name="com.voiceworks/ox-messenger"]').prepend(
                    $('<span class="messenger--badge-unread">').append(
                        $('<p>').text('0')
                    )
                );
                $('.messenger--badge-unread').hide();
            }

            $('.messenger--badge-unread p').text(unreadCount);
            $('.messenger--badge-unread').show();
        },

        updateAppUnreadActiveConversation: function () {
            var activeConversation = this.get(this.currentConversationId);

            if (activeConversation) {
                activeConversation.set('unreadMessages', 0);
                activeConversation.set('scroll', true);
            }
        },

        setSortMethod: function (field, direction) {
            this.sortMethod = field;
            this.sortDirection = direction;
        }

    });

    return ConversationCollection;
});