/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/conversation/models/message', [
    'io.ox/mail/detail/links',
    'com.voiceworks/ox-messenger/file/models/file-desc',
    'com.voiceworks/ox-messenger/roster/main'
], function (Links, FileDesc,  Roster) {
    'use strict';

    /**
     * Message Model
     */
    var MessageModel = Backbone.Model.extend({

        type: 'base',

        defaults: {
            from: '',
            message: '',
            stamp: '',
            subject: '',
            username: '',
            roomId: '',
            timestamp: 0,
            roomJid: null,
            isHistory: false,
            user: null,
            fileDesc: null
        },

        initialize: function (options) {
            this.conversation = options.conversation;

            if (options.username) {
                this.user = Roster.getUserById(options.username);
            }

            // Setup the timestamp related attributes
            if (options.timestamp) {
                this.set('timestamp', options.timestamp);
                this.set('isHistory', true);
            } else {
                this.set('timestamp', new Date().getTime());
                this.set('isHistory', false);
            }

            // Make sure fileDesc is a model
            if(this.get('fileDesc') && !(this.get('fileDesc') instanceof FileDesc)){
                this.set('fileDesc', new FileDesc(this.get('fileDesc')));
            }
        },

        is: function (type) {
            return this.type === type;
        },

        getRoomJid: function () {
            return this.get('roomId') + '@' + this.get('host');
        },

        getSenderJid: function () {
            return this.get('username');
        },

        getFileDescAsObject: function() {
            if(this.get('fileDesc'))
                return this.get('fileDesc').toObject();
            else
                return null;
        }
    });

    return MessageModel;
});
