﻿/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/conversation/models/messagearchive', [
    'com.voiceworks/ox-messenger/conversation/models/message/factory',
    'com.voiceworks/ox-messenger/util'
], function (MessageFactory, Util) {
    'use strict';

    /**
     * MessageArchive Model
     */
    var MessageArchiveModel = Backbone.Model.extend({

        defaults: {
            jid: null,
            /** @type {Strophe.ArchivedCollection} */
            archive: null,
            startDate: null,
            fetched: false,
            user: null
        },

        getMessageModels: function (callback) {
            this.get('archive').retrieveMessages(null, function (messages, rsm) {
                this.getMessageModelsCallback(messages, rsm, callback);
            }.bind(this));
        },

        /**
         * @param {Array.<Strophe.ArchivedCollection>} messages
         * @param {Strophe.RSM} rsm
         * @param {function(Array.<MessageArchiveModel>)} callback
         */
        getMessageModelsCallback: function (messages, rsm, callback) {
            var archive = this.get('archive');
            var ret = messages.map(function (msg) {
                var from = Util.getBareJid(msg.from),
                    historyObject = {
                        from: from,
                        username: from,
                        message: msg.body,
                        timestamp: msg.timestamp.getTime(),
                        isHistory: true,
                        user: this.get('user'),
                        fileDesc: msg.fileDesc
                    };

                return MessageFactory.create(historyObject);
            }, this);

            // TODO this may be unnecessary, check and remove if it's
            ret = _.sortBy(ret, function (item) {
                return item.get('timestamp');
            });

            this.set('fetched', true);

            return callback(ret);
        },
        
        getTime: function() {
            return this.get('startDate').getTime();
        }
    });

    return MessageArchiveModel;
});
