/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/conversation/models/messagecontainer', [
    'com.voiceworks/ox-messenger/conversation/models/messagecollection'
], function (MessageCollection) {
    'use strict';

    /**
     * Message Container
     */
    var MessageGroup = Backbone.Model.extend({
        klassName: 'MessageGroup',

        defaults: {
            conversation: null,
            messages: null,
            senderUser: null,
            fileDesc: null
        },

        messages: null,

        // TODO first param is data, not options (results in a duplicate allocation)
        initialize: function (options) {
            MessageGroup.__super__.initialize.apply(this, arguments);

            //this.conversation = options ? options.conversation : null;
            this.messages = new MessageCollection();
            // TODO getter
            this.senderUser = options ? options.senderUser : null;
        },

        /**
         *
         * @param message
         * @param {boolean} prepend Only for history messages to preserve the order of
         * msgs with the same timestamp.
         */
        addMessage: function (message) {
            this.messages.add(message);
        },


        /**
         * Make this group potentially dirty because of the passed message.
         *
         * This can happen when splitting a group.
         *
         * @param message
         * @return {boolean}
         */
        setDirty: function(message) {
            if (!this.messages.contains(message)) {
                return false;
            }
            // Transfer the ownership of this group to other messages.
            if (this.messages.length > 1) {
                this.messages.remove(message);
                message.group = null;
                return true;
            }
            return false;
        }

    });

    return MessageGroup;
});
