/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/conversation/models/messagecontainercollection', [
    'com.voiceworks/ox-messenger/conversation/models/messagecontainer'
], function (MessageGroupModel) {
    'use strict';

    /**
     * Message Container Collection
     */
    var MessageGroupCollection = Backbone.Collection.extend({
        klassName: 'MessageGroupCollection',
        //model: MessageGroupModel,
        messagesCollection: null,

        initialize: function (models, options, messagesCollection) {
            //Backbone.Collection.prototype.initialize.call(this);

            assert(messagesCollection);

            this.messagesCollection = messagesCollection;
            this.bindToMessageCollection();
            this.parseMessageCollection();
        },

        bindToMessageCollection: function() {
            this.listenTo(this.messagesCollection, 'add', function(model) {
                // Set sibling groups as dirty, as msgs can come in any order
                var index = this.messagesCollection.models.indexOf(model);
                var previous = this.messagesCollection.models[index-1];
                var next = this.messagesCollection.models[index+1];

                // Split a group if needed
                if (next && previous && previous.group && previous.group.senderUser != model.user) {
                    var i = index + 1;
                    while (next && next.group == previous.group) {
                        next.group.setDirty(next);
                        next = this.messagesCollection.models[++i];
                    }
                }
                // TODO Trigger only when rendered and on the first render.
                this.parseMessageCollection(index);
            });
        },

        parseMessageCollection: function(positionIndex) {
            this.messagesCollection.forEach(function(message, index) {
                if (!message.group) {
                    // TODO move fileDesc check and others to
                    //   MessageGroup::canMerge method
                    var isFile = message.has('fileDesc');
                    if (!isFile && index && this.messagesCollection.at(index-1).group
                            && this.messagesCollection.at(index-1).group.senderUser == message.user
                            && !this.messagesCollection.at(index-1).group.get('fileDesc')) {
                        message.group = this.messagesCollection.at(index-1).group;
                        message.group.addMessage(message, positionIndex);
                    } else if (!isFile && this.messagesCollection.at(index+1)
                            && this.messagesCollection.at(index+1).group
                            && this.messagesCollection.at(index+1).group.senderUser == message.user
                            && !this.messagesCollection.at(index+1).group.get('fileDesc')) {
                        message.group = this.messagesCollection.at(index+1).group;
                        message.group.addMessage(message, positionIndex);
                    } else {
                        message.group = new MessageGroupModel({
                            senderUser: message.user,
                            fileDesc: message.get('fileDesc')
                        });
                        message.group.addMessage(message, positionIndex);
                    }
                }
                // Re-add flushed groups
                if (!this.contains(message.group)) {
                    this.add(message.group, {
                        sort: false
                    });
                }
            }, this);

            // Re-sort groups after all the potential changes.
            this.sort();
        },

        comparator: function (group) {
            var first = group.messages.first();
            
            // Group messages are already sorted
            return first.get('timestamp');
        }
    });

    return MessageGroupCollection;
});
