/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/conversation/toolbar', [
    'io.ox/core/extensions',
    'io.ox/core/extPatterns/links',
    'io.ox/backbone/mini-views/dropdown',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/actions',
    'gettext!com.voiceworks/ox-messenger',
    'less!io.ox/mail/style'
], function (ext, links, Dropdown, Aggregator, Actions, gt) {

    'use strict';

    // define links for classic toolbar
    var point = ext.point('com.voiceworks/ox-messenger/classic-toolbar/links');

    var meta = {
        'start-groupconversation': {
            prio: 'hi',
            mobile: 'hi',
            label: gt('New conversation'),
            title: gt('Start a new conversation with one or more buddies'),
            drawDisabled: true,
            ref: 'com.voiceworks/ox-messenger/actions/start-group-conversation'
        },

        'add-buddy': {
            prio: 'hi',
            mobile: 'hi',
            label: gt('Add buddy'),
            title: gt('Add a buddy to your buddylist'),
            drawDisabled: true,
            ref: 'com.voiceworks/ox-messenger/actions/addbuddy'
        },

        'invite-guest': {
            prio: 'hi',
            mobile: 'hi',
            label: gt('Invite guest'),
            title: gt('Invite a temporary guest user'),
            drawDisabled: true,
            ref: 'com.voiceworks/ox-messenger/actions/inviteguest'
        },

        'call-history': {
            prio: 'hi',
            mobile: 'hi',
            label: gt('Call history'),
            title: gt('Show a list of all calls'),
            drawDisabled: true,
            ref: 'com.voiceworks/ox-messenger/actions/call-history'
        },

        'delete': {
            prio: 'hi',
            mobile: 'lo',
            icon: 'fa fa-trash-o',
            label: gt('Delete conversation'),
            title: gt('Delete conversation'),
            drawDisabled: true,
            ref: 'com.voiceworks/ox-messenger/actions/conversation/delete'
        }
    };

    // transform into extensions

    var index = 0;

    _(meta).each(function (extension, id) {
        extension.id = id;
        extension.index = (index += 100);
        point.extend(new links.Link(extension));
    });

    ext.point('com.voiceworks/ox-messenger/classic-toolbar').extend(new links.InlineLinks({
        attributes: {},
        classes: '',
        forcelimit: true, // always use drop-down
        index: 200,
        id: 'toolbar-links',
        ref: 'com.voiceworks/ox-messenger/classic-toolbar/links'
    }));

    // view dropdown
    ext.point('com.voiceworks/ox-messenger/classic-toolbar').extend({
        id: 'view-dropdown',
        index: 10000,
        draw: function (baton) {

            if (_.device('small')) {
                return;
            }

            //#. View is used as a noun in the toolbar. Clicking the button opens a popup with options related to the View
            var dropdown = new Dropdown({ model: baton.app.props, label: gt('View'), tagName: 'li' })
                .header(gt('Layout'))
                .option('layout', 'vertical', gt('Vertical'))
                .option('layout', 'compact', gt('Compact'))
                .divider()
                .header(gt('Options'))
                .option('checkboxes', true, gt('Checkboxes'))
                .option('contactPictures', true, gt('Contact pictures'));

            this.append(
                dropdown.render().$el.addClass('pull-right').attr('data-dropdown', 'view')
            );

        }
    });

    // classic toolbar
    var toolbar = $('<ul class="classic-toolbar" role="menu">');

    var updateToolbar = _.debounce(function (list) {
        //if (!list) return;
        // remember if this list is based on a single thread
        //var isThread = list.length === 1 && /^thread\./.test(list[0]);
        // resolve thread
        //list = api.threads.resolve(list);
        //if (list.length === 0) isThread = false;
        // extract single object if length === 1
        //list = list.length === 1 ? list[0] : list;
        // draw toolbar

        var selectedCollection = new Backbone.Collection();
        _(list).each(function (selectedItem) {
            selectedCollection.add(new Backbone.Model().set('id', selectedItem));
        });

        var baton = ext.Baton({ $el: toolbar, app: this, data: selectedCollection });
        ext.point('com.voiceworks/ox-messenger/classic-toolbar').invoke('draw', toolbar.empty(), baton);
    }, 10);

    ext.point('com.voiceworks/ox-messenger/mediator').extend({
        id: 'toolbar',
        index: 10000,
        setup: function (app) {
            if (_.device('small')) {
                return;
            }
            app.getWindow().nodes.body.addClass('classic-toolbar-visible').prepend(
                toolbar = $('<ul class="classic-toolbar" role="menu">')
            );
            app.updateToolbar = updateToolbar;
        }
    });

    ext.point('com.voiceworks/ox-messenger/mediator').extend({
        id: 'update-toolbar',
        index: 10200,
        setup: function (app) {
            if (_.device('small')) {
                return;
            }
            app.updateToolbar();
            // update toolbar on selection change as well as any model change (seen/unseen flag)
            Aggregator.on('conversationlist:selection:change', function (selection) {
                app.updateToolbar(selection);
            });
        }
    });

    ext.point('com.voiceworks/ox-messenger/view-options').extend({
        id: 'sort',
        index: 100,
        draw: function (batton) {
            this.data('view')
                .option('sort', 'timestamp', gt('Date'))
                .option('sort', 'sortName', gt('Name'));
        }
    });

    ext.point('com.voiceworks/ox-messenger/view-options').extend({
        id: 'order',
        index: 200,
        draw: function () {
            this.data('view')
                .divider()
                .option('order', 'asc', gt('Ascending'))
                .option('order', 'desc', gt('Descending'));
        }
    });

    ext.point('com.voiceworks/ox-messenger/list-view/toolbar/top').extend({
        id: 'select-all',
        index: 100,
        draw: function (baton) {
            this.append(
                $('<a href="#" class="toolbar-item select-all" role ="checkbox" aria-checked="false" tabindex="1">').append(
                    $('<i class="fa fa-square-o" aria-hidden="true">'),
                    $.txt(gt('Select all'))
                )
                .on('click', { baton: baton }, toggleSelection)
                .on('dblclick', function (e) {
                    e.stopPropagation();
                })
                .on('keydown', { baton: baton }, toggleSelection)
            );

            var i = this.find('.select-all > i');

            baton.view.listView.on({
                'selection:all': function () {
                    toggleControl(i, true);
                },
                'selection:subset': function () {
                    toggleControl(i, false);
                }
            });
        }
    });

    function toggleControl(i, state) {
        i.attr('class', state ? 'fa fa-check-square-o' : 'fa fa-square-o').parent().attr('aria-checked', state);
    }

    function toggleSelection(e) {
        if (e.type === 'click' || e.keyCode === 32) {
            e.preventDefault();
            var i = $(this).find('i'), selection = e.data.baton.app.listControl.listView.selection;
            if (i.hasClass('fa-check-square-o')) {
                selection.selectNone();
                // selectNone is not triggering the 'CHANGE' event properly
                Aggregator.trigger('conversationlist:selection:change', []);
            }
            else {
                selection.selectAll();
            }
        }
    }

    ext.point('com.voiceworks/ox-messenger/list-view/toolbar/top').extend({
        id: 'dropdown',
        index: 1000,
        draw: function (baton) {

            var dropdown = new Dropdown({
                //#. Sort options drop-down
                label: gt.pgettext('dropdown', 'Sort by'),
                model: baton.app.props
            });

            ext.point('com.voiceworks/ox-messenger/view-options').invoke('draw', dropdown.$el, baton);
            this.append(dropdown.render().$el.addClass('grid-options toolbar-item pull-right').on('dblclick', function (e) {
                e.stopPropagation();
            }));
        }
    });
});
