/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/conversation/views/call', [
    'io.ox/core/extensions',
    'io.ox/core/extPatterns/actions',
    'io.ox/core/notifications',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/core/session',
    'com.voiceworks/ox-messenger/core/views/disposable',
    'com.voiceworks/ox-messenger/core/views/timer',
    'com.voiceworks/ox-messenger/util',
    'com.voiceworks/ox-messenger/conversation/views/stream',
    'com.voiceworks/ox-messenger/core/views/partials/callpart',
    'static/3rd.party/bigscreen/bigscreen.min.js',
    'gettext!com.voiceworks/ox-messenger',
    'com.voiceworks/ox-messenger/core/vendor/hoverIntent'
], function (ext, extPatternActions, CoreNotifications, Aggregator, Session, DisposableView, TimerView, Util,
             StreamView, CallPart, BigScreen, gt) {
    'use strict';

    /**
     * Call View
     */
    var CallView = DisposableView.extend({
        className: 'messenger--table__row  messenger--table__row--conversation-call  messenger--is-hidden',

        // Attributes
        controlHoverTimout: false,
        fullscreenActive: false,

        // Controls events
        events: {
            'click .messenger--btn--call-action--fullscreen'        : 'toggleFullscreen',
            'click .messenger--btn--call-action--leave-fullscreen'  : 'toggleFullscreen',
            'click .messenger--btn--call-action--mute-video'        : 'toggleVideoMute',
            'click .messenger--btn--call-action--mute-audio'        : 'toggleAudioMute',
            'click .messenger--btn--call-action--hangup'            : 'hangup',
            'click .messenger--btn--call-action--group-call-invite' : 'groupCallInvite'
        },

        initialize: function () {
            DisposableView.prototype.initialize.apply(this, arguments);

            this.listenTo(this.model.callParticipants, 'add', this.createCallStream);
            this.listenTo(this.model.callParticipants, 'remove', this.removeCallStream);
            this.listenTo(this.model.callParticipants, 'reset', this.resetCallStreams);

            BigScreen.onchange = $.proxy(this.bigScreenHandler, this);

            Aggregator.on('call:startHangUpTimeout', $.proxy(this.startHangUpTimeoutHandler, this));
        },

        // ------------------------------------------- Stream Methods ----------------------------------

        createStreams: function (media) {
            var that = this;

            this.$el.find('.messenger--call-group-video').removeClass('messenger--is-hidden');

            // Create 1 stream per participant
            if (this.$el.find('.messenger--call-group-video__participant').length === 0) {
                this.model.callParticipants.each(function (participant) {
                    that.createCallStream(participant);
                });
            }

            // When Video Call: Show local stream
            if (Session.user.call.get('type') === 'groupvideo' ||
                Session.user.call.get('type') === 'video') {
                this.createLocalStream(media);
            }

            if (Session.user.callActive) {
                Aggregator.trigger('call:showView');
            }
        },

        createLocalStream: function (media) {
            // Clean old video
            $('#videoLocal').remove();

            media.local.setAttribute('width', '200px');
            media.local.setAttribute('autoplay', true);
            media.local.setAttribute('id', 'videoLocal');

            this.$el.find('.messenger--call-mycam')
                .empty()
                .removeClass('messenger--is-hidden')
                .append(media.local);
        },

        createCallStream: function (participant) {
            if (!participant.call.get('type')){
                return;
            }

            if (participant.call.get('type') !== 'groupaudio') {
                if (Util.isSameJid(participant.id, Session.user.id)) {
                    return;
                }
            }

            var streamView = new StreamView({
                model: participant,
                title: (participant.call.get('type') === 'groupaudio') ?
                            this.model.get('title') :
                            participant.getDisplayName()
            });

            var streamsElem = this.$el.find('.messenger--call-group-video__streams'),
                stream = streamView.render().$el;

            streamsElem.append(stream);

            if (participant.call.get('type') === 'groupaudio' && !Util.isSameJid(participant.id, Session.user.id)){
                stream.addClass('messenger--is-hidden');
            }
        },

        removeCallStream: function (participant) {
            var streamFilter = '#' + participant.getStreamId();

            this.$el.find(streamFilter).remove();
        },

        resetCallStreams: function () {
            this.$el.find('.messenger--call-group-video__streams').children().remove();
        },

        // ----------------------------------- Display Methods ---------------------------------

        show: function () {
            this.$el.removeClass('messenger--is-hidden');
        },

        hide: function () {
            this.$el.addClass('messenger--is-hidden');
        },

        hideVideo: function () {
            this.hide();

            this.$el.find('.messenger--call-video').addClass('messenger--is-hidden').children().remove();
            this.$el.find('.messenger--call-mycam').addClass('messenger--is-hidden').children().remove();

            if (this.fullscreenActive) {
                BigScreen.exit();
            }
         },

        hideAudio: function () {
            this.hide();

            this.$el.removeClass('is-audio');
            this.$el.find('.messenger--call-audio').children().detach();
        },

        showControls: function () {
            clearTimeout(this.controlHoverTimeout);

            this.$el.find('.messenger--call-controls').addClass('is-active');

            // Hide buttons for audio call
            if (!Session.user.call.isCallSessionVideo()) {
                this.$el.find(
                    '.messenger--btn--call-action--mute-video, .messenger--btn--call-action--fullscreen'
                ).addClass('messenger--is-hidden');

            // Otherwise (video) show all controls
            } else {
                this.$el.find(
                    '.messenger--btn--call-action--mute-video, .messenger--btn--call-action--fullscreen'
                ).removeClass('messenger--is-hidden');
            }

            //if (Session.user.call.isGroup()) {
                this.$el.find(
                    '.messenger--btn--call-action--group-call-invite'
                ).removeClass('messenger--is-hidden');
            //}
        },

        hideControls: function () {
            var self = this;
            this.controlHoverTimeout = setTimeout(function () {
                if (self.$el) {
                    self.$el.find('.messenger--call-controls').removeClass('is-active');
                }
            }, 1000);
        },

        showCallConnecting: function (show) {
            if (show){
                this.$el.find('.messenger--throbber.call--connecting').removeClass('messenger--is-hidden');
            }
            else {
                this.$el.find('.messenger--throbber.call--connecting').addClass('messenger--is-hidden');
            }
        },

        render: function () {
            this.$el.children().remove();
            this.$el.prevObject = null;

            this.$el.append(
                $('<div class="messenger--table__cell  messenger--table__cell--conversation-call">').append(
                    $('<div class="messenger--table">').append(
                        $('<div class="messenger--table__row  messenger--table__row--call">').append(
                            $('<div class="messenger--call">').append(
                                $('<div class="messenger--call-group-video  messenger--is-hidden">').append(
                                    $('<div class="messenger--table  messenger--call-group-video__streams">')
                                )
                            )
                        )
                    ),
                    $('<div class="messenger--call-mycam  messenger--is-hidden">'),
                    $('<div class="messenger--call-controls">').append(
                        $('<button class="btn messenger--btn--call-action messenger--btn--call-action--group-call-invite  messenger--is-hidden" data-action="group-call-invite" tabindex="2" type="button">').attr({
                            'title': gt('Invite user to group call')
                        }).append(
                            $('<i class="fa fa-plus  messenger--fa-plus--large"></i>')
                        ),
                        $('<button class="btn messenger--btn--call-action messenger--btn--call-action--mute-video">').attr({
                            'title': gt('Disable camera'),
                            'data-action': 'video',
                            'tabindex': '2',
                            'type': 'button'
                        }).append(
                            $('<i class="fa fa-eye-slash"></i>')
                        ),
                        $('<button class="btn messenger--btn--call-action messenger--btn--call-action--hangup">').attr({
                            'title': gt('Hangup call'),
                            'data-action': 'hangup',
                            'tabindex': '3',
                            'type': 'button'
                        }).append(
                            $('<i class="fa fa-phone"></i>')
                        ),
                        $('<button class="btn messenger--btn--call-action messenger--btn--call-action--mute-audio">').attr({
                            'title': gt('Mute microphone'),
                            'data-action': 'mute',
                            'tabindex': '4',
                            'type': 'button'
                        }).append(
                            $('<i class="fa fa-microphone-slash"></i>')
                        ),
                        $('<button class="btn messenger--btn--call-action messenger--btn--call-action--fullscreen">').attr({
                            'title': gt('View in full screen'),
                            'data-action': 'fullscreen',
                            'tabindex': '5',
                            'type': 'button'
                        }).append(
                            $('<i class="fa fa-expand"></i>')
                        )
                    )
                )
            );

            this.$el.hoverIntent(
                $.proxy(this.showControls, this),
                $.proxy(this.hideControls, this)
            );

            return this;
        },

        // ---------------------------------------- Controls events -------------------------------

        toggleFullscreen: function () {
            var fullscreenContainer = this.$el.parents('#conversation-fullscreen-container');
            BigScreen.toggle(fullscreenContainer[0]);
        },

        toggleVideoMute: function () {
            var muted = Session.user.call.get('callSession').toggleVideoMute();
            if (muted) {
                $('.messenger--btn--call-action--mute-video').addClass('active');
            } else {
                $('.messenger--btn--call-action--mute-video').removeClass('active');
            }
            Session.user.call.set('videoMuted', muted);
        },

        toggleAudioMute: function () {
            var muted = Session.user.call.get('callSession').toggleAudioMute();
            if (muted) {
                $('.messenger--btn--call-action--mute-audio').addClass('active');
            } else {
                $('.messenger--btn--call-action--mute-audio').removeClass('active');
            }
            Session.user.call.set('audioMuted', muted);
        },

        hangup: function () {
            Aggregator.trigger('call:hangup', {
                conversation: this.model
            });
        },

        groupCallInvite: function () {
            if (!this.model.canJoinGroupCall(Session.user.call.get('type'))) {
                CoreNotifications.yell('error', gt('It is not possible to add a new participant, you have reached the maximum number of group call participants.'));
                return;
            }

            var baton = new ext.Baton({
                data: this.model,
                isGroupCallInvite: true
            });

            if (this.fullscreenActive) {
                BigScreen.exit();
            }

            extPatternActions.invoke('com.voiceworks/ox-messenger/actions/addparticipant', this, baton);
        },

        // ------------------------------------ Handler Methods ----------------------------

        startHangUpTimeoutHandler: function (id) {
            if (this.model && this.model.id === id) {
                var hangupElem = this.$el.find('.messenger--btn--call-action--hangup'),
                    timeoutTime = 3000;

                hangupElem.prop('disabled', true);

                setTimeout(function () {
                    hangupElem.prop('disabled', false);
                }, timeoutTime);
            }
        },

        bigScreenHandler: function () {
            var fullscreenContainer = this.$el.parents('#conversation-fullscreen-container');

            if (!this.fullscreenActive) {
                fullscreenContainer.addClass('is-fullscreen');
                fullscreenContainer.find('.messenger--table__row--chat-messages').addClass('messenger--is-hidden');
                fullscreenContainer.find('.messenger--table__row--chat-input').addClass('messenger--is-hidden');
                fullscreenContainer.find('.messenger--table__cell--participants-list').addClass('messenger--is-hidden');
                fullscreenContainer.find('.fa-expand').removeClass('fa-expand').addClass('fa-compress');

                this.fullscreenActive = true;
            }
            else {
                fullscreenContainer.removeClass('is-fullscreen');
                fullscreenContainer.find('.messenger--table__row--chat-messages').removeClass('messenger--is-hidden');
                fullscreenContainer.find('.messenger--table__row--chat-input').removeClass('messenger--is-hidden');
                fullscreenContainer.find('.messenger--table__cell--participants-list').removeClass('messenger--is-hidden');
                fullscreenContainer.find('.fa-compress').removeClass('fa-compress').addClass('fa-expand');

                this.fullscreenActive = false;
            }
        }
    });

    return CallView;
});
