/**
/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/conversation/views/stream', [
    'com.voiceworks/ox-messenger/core/views/disposable',
    'com.voiceworks/ox-messenger/core/views/partials/callpart',
    'com.voiceworks/ox-messenger/core/views/timer',
    'com.voiceworks/ox-messenger/aggregator',
    'gettext!com.voiceworks/ox-messenger'
], function (DisposableView, CallPart, TimerView, Aggregator, gt) {
    'use strict';

    var StreamView = DisposableView.extend({
        className: 'messenger--table__cell  messenger--table__cell--centered  messenger--call-group-video__participant',

        initialize: function (options) {
            DisposableView.prototype.initialize.apply(this, arguments);

            // Init vars
            this.id = this.model.id;
            this.title = options.title || this.model.getDisplayName();

            // Listen to model changes
            this.listenTo(this.model.call, 'change:callState', this.stateCheck);
            this.listenTo(this.model, 'change:streamFrozen', this.streamFrozenHandler);

            Aggregator.on('call:showView', $.proxy(this.showCallViewHandler, this));
        },

        // --------------------------------------- Render Methods --------------------------------------------

        render: function () {
            var infoDiv = $('<div class="messenger--call-info">');

            // Clean
            this.$el.children().remove();
            this.$el.prevObject = null;

            // Create stream id
            this.$el.attr('id', this.model.getStreamId());

            // If it is a Video Stream
            if (this.model.call.get('type') === 'groupvideo' ||
                this.model.call.get('type') === 'video') {
                // Add group-video specific Elements
                this.$el.append(
                    $('<div class="messenger--call-group-video__video-container  messenger--is-hidden">').append(
                        $('<div class="messenger--call-group-video__frozen  messenger--is-hidden">').append(
                            $('<i class="fa fa-spinner fa-spin">')
                        ),
                        $('<div class="messenger--call-group-video__video">')
                    ),
                    $('<div class="messenger--call-group-video__video-name  messenger--text-wrap  messenger--is-hidden">').text(
                        this.title
                    )
                );

                // Setup info Element
                infoDiv.addClass('messenger--call-group-video__call-info');
                // Stream avatar
                infoDiv.append(
                    $('<div class="messenger--avatar  messenger--avatar--extra-large  messenger--call-group-video__avatar">').css(
                        {'background-image': 'url(' + this.model.getImageUrl() + ')'}
                    )
                );
            }
            // If it is an Audio Stream
            else {
                // Setup info Element
                infoDiv.addClass('messenger--call-group-audio__call-info');
                // Stream avatar
                infoDiv.append(
                    $('<div class="messenger--avatar  messenger--avatar--extra-large  messenger--call-group-video__avatar  messenger--avatar--icon  messenger--avatar--icon--large">').append(
                        $('<div class="fa fa-group"></div>')
                    )
                );
            }

            // Finish info Element
            infoDiv.append(
                $('<div class="messenger--call-group__name  messenger--text-wrap  messenger--font--bold">').text(this.title),
                $('<div class="messenger--call-group__status  messenger--font--light">'),
                CallPart.getConnectingAnimation()
            );

            // Add info element to View
            this.$el.append(infoDiv);

            // Update View
            this.stateCheck();

            return this;
        },

        showVideoContainer: function (show) {
            if (show) {
                this.$el.find('.messenger--call-group-video__video-container').removeClass('messenger--is-hidden');
            }
            else {
                this.$el.find('.messenger--call-group-video__video-container').addClass('messenger--is-hidden');
            }
        },

        showVideoName: function (show) {
            if (show) {
                if (!this.$el.find('.messenger--call-info').hasClass('messenger--is-hidden')) {
                    this.$el.find('.messenger--call-info').addClass('messenger--is-hidden');
                }
                this.$el.find('.messenger--call-group-video__video-name').removeClass('messenger--is-hidden');
            }
            else {
                this.$el.find('.messenger--call-group-video__video-name').addClass('messenger--is-hidden');
            }
        },

        showInfo: function (show) {
            if (show) {
                if (!this.$el.find('.messenger--call-group-video__video-name').hasClass('messenger--is-hidden')) {
                    this.$el.find('.messenger--call-group-video__video-name').addClass('messenger--is-hidden');
                }
                this.$el.find('.messenger--call-info').removeClass('messenger--is-hidden');
            }
            else {
                this.$el.find('.messenger--call-info').addClass('messenger--is-hidden');
            }
        },

        showCallConnecting: function (show) {
            if (show) {
                this.$el.find('.messenger--throbber.call--connecting').removeClass('messenger--is-hidden');
            }
            else {
                this.$el.find('.messenger--throbber.call--connecting').addClass('messenger--is-hidden');
            }
        },

        setStatusLabel: function (txt) {
            if ((this.model.call.get('type') === 'groupaudio' || this.model.call.get('type') === 'audio') &&
                txt === gt('Is in the call')) {
                return;
            }

            this.$el.find('.messenger--call-group__status').text(txt);
        },

        // -------------------------- State methods ---------------------------------
        stateCheck: function () {
            var callState = this.model.call.get('callState');

            switch (callState) {
                case 'idle':
                case 'calling':
                    this.stateIsIdle();
                    break;
                case 'in-progress':
                    this.stateIsInProgress();
                    break;
                case 'group-initiated':
                    this.stateIsGroupInitiated();
                    break;
                case 'answered':
                    this.stateIsAnswered();
                    break;
            }
        },

        // idle: hangup
        // hide all
        // show nothing
        stateIsIdle: function () {
            this.$el.find('.messenger--call-group-video__video-container video').remove();
            this.showVideoContainer(false);

            // this is for calling:
            this.showInfo(true);
            if (this.model.call.get('type') !== 'groupaudio') {
                this.setStatusLabel(this.model.call.getCallStateDisplayName());
            }
            this.model.set('lastCallFrozen', this.model.get('streamFrozen'));
            this.model.set('streamFrozen', false);
        },

        // in-progress: not yet answered
        // Show Ringing status
        stateIsInProgress: function () {
            this.showVideoContainer(false);
            this.showInfo(true);
            this.showVideoName(false);
            if (this.model.call.get('type') !== 'groupaudio') {
                this.setStatusLabel(this.model.call.getCallStateDisplayName());
            }
        },

        // group-initiated: Only when it's my stream on a group audio call
        stateIsGroupInitiated: function () {
            // Only processed if it's an group audio call
            if (this.model.call.get('type') === 'groupaudio') {
                var timerView = new TimerView({
                        model: this.model.call.timer
                    });

                // Update the status label while waiting for media
                this.setStatusLabel(this.model.call.getCallStateDisplayName());

                // TODO: Handle 'canplay' event as we do with video calls
                // Firefox & Chrome handle that event in a different way and the change must be done on UCConnect level
                // Hide call connecting animation
                this.showCallConnecting(false);

                // Add the timer as status
                this.$el.find('.messenger--call-group__status')
                    .empty()
                    .append(timerView.render().$el);
            }
        },

        // answered: call started
        // show stream and add video to stream, and show timer
        // hide icon and status
        stateIsAnswered: function () {
            var callSession = this.model.call.get('callSession');

            if(callSession) {
                this.media = callSession.getMediaCanvas();
                var media = this.media,
                    videoElem = this.$el.find('.messenger--call-group-video__video'),
                    streamId = 'video-stream-' + this.model.id.replace(/[@|.]/g, '_'),
                    tmpVideoStream = $('#' + streamId),
                    onCanPlay = function () {
                        // When Group Video Call
                        this.onCanPlayStream();
                        // Listen to the 'canplay' to wait for media
                        media.remote.removeEventListener('canplay', onCanPlay);
                    };

                // There is a video saved, and it's a MUC
                if (tmpVideoStream.length) {
                    media.remote = tmpVideoStream.get(0);
                    tmpVideoStream.remove();
                }
                media.remote.setAttribute('id', streamId);
                media.remote.setAttribute('class', 'groupStreamVideo');

                if (media.remote) {
                    if (this.model.call.get('type') === 'groupvideo' ||
                        this.model.call.get('type') === 'video') {
                        // Progress handler
                        this.progressHandler = $.proxy(function () {
                            var tmpTime        = new Date().getTime(),
                                tmpFrozenLimit = 5000,
                                frozen         = this.model.get('streamFrozen'),
                                currentTime    = this.model.get('streamCurrentTime');

                            // If the stream is frozen, send a keyframe request to the media server
                            if ((tmpTime - currentTime) > tmpFrozenLimit && !frozen && currentTime !== 0) {
                                this.model.call.refreshStream();
                                this.model.set('streamFrozen', true);
                                this.model.set('streamFrozen', true);
                            }
                            // If there's n
                            else if ((tmpTime - currentTime) <= tmpFrozenLimit && frozen && currentTime !== 0) {
                                this.model.set('streamFrozen', false);
                            }
                        }, this);
                        // TimeUpdate handler
                        this.timeupdateHandler = $.proxy(function () {
                            if (this.model) {
                                this.model.set('streamCurrentTime', new Date().getTime());
                            }
                        }, this);

                        // Initilize the frozen timestamp
                        this.timeupdateHandler();

                        // Listen to the media events
                        $(media.remote)
                            .on('progress', this.progressHandler)
                            .on('timeupdate', this.timeupdateHandler);
                    }
                }

                videoElem.empty().append(media.remote);

                if (this.model.call.get('type') === 'groupvideo' ||
                    this.model.call.get('type') === 'video') {

                    // Listen to the 'canplay' to wait for media
                    media.remote.addEventListener('canplay', $.proxy(onCanPlay, this), false);
                    if (media.remote.paused) {
                        this.setStatusLabel(this.model.call.getCallStateDisplayName());
                        setTimeout(function () {
                            media.remote.play();
                        }, 100);
                    }
                } else {
                    this.onCanPlayStream();
                }
            }
        },

        onCanPlayStream: function () {
            if (this.model === null) {
                return;
            }
            if (this.model.call.get('type') === 'groupvideo' ||
                this.model.call.get('type') === 'video') {
                this.showVideoContainer(true);
                this.showVideoName(true);
                this.showInfo(false);

                // When Group Audio Call
            } else if (this.model.call.get('type') === 'groupaudio' ||
                this.model.call.get('type') === 'audio') {
                this.setStatusLabel(this.model.call.getCallStateDisplayName());
                this.showInfo(true);
                this.showCallConnecting(false);

                // TODO: First help for showing double avatar icon
                var infoElement =  $(this.$el.parent().parent().parent().find('.messenger--call-info')[0]);
                if (!infoElement.hasClass('messenger--is-hidden')) {
                    $(this.$el.parent().parent().parent().find('.messenger--call-info')[0]).addClass('messenger--is-hidden');
                }
            }
        },

        // -------------------------------------- Event handlers -------------------------------

        streamFrozenHandler: function (event) {
            var frozen = this.model.get('streamFrozen');

            if (frozen) {
                this.$el.find('.messenger--call-group-video__frozen').removeClass('messenger--is-hidden');
            } else {
                this.$el.find('.messenger--call-group-video__frozen').addClass('messenger--is-hidden');
            }
        },

        showCallViewHandler: function() {
            if (this.model === null) {
                return;
            }

            // When Video Call
            if (this.model.call.get('type') === 'groupvideo' ||
                this.model.call.get('type') === 'video') {
                if (this.model.call.get('callState') === 'answered') {
                    this.showVideoContainer(true);
                    this.showVideoName(true);
                    this.showInfo(false);
                }
            }
            // When Audio Call
            else if (this.model.call.get('type') === 'groupaudio') {
                this.showInfo(true);
                this.showCallConnecting(false);
            }

            var self = this, call = this.model.call;

            setTimeout(function () {
                    //console.debug('check call:unsuccessful', call, call.get('callState'), call.get('callSession').display_name);

                    if (call && (typeof call.get('callSession') === 'undefined' || call.get('callSession') === null ) ) {
                        console.debug('No call session triggered ');
                        Aggregator.trigger('call:unsuccessful', self.id);
                    }
                    else if (call && call.get('callSession').display_name === 'recv' && call.get('callState') !== 'answered') {
                        Aggregator.trigger('call:unsuccessful', self.id);
                    }
                }
                ,70000);
        },

        dispose: function() {
            // remove Progress Listener
            if (this.media) {
                $(this.media.remote)
                    .off('progress', this.progressHandler)
                    .off('timeupdate', this.timeupdateHandler);
                delete this.media;
            }

            DisposableView.prototype.dispose.call(this); // the super call
        }
    });

    return StreamView;
});
