/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/core/badge', [
    'io.ox/core/extensions',
    'io.ox/core/extPatterns/stage',
    'io.ox/core/capabilities',
    'com.voiceworks/ox-messenger/core/session',
    'com.voiceworks/ox-messenger/roster/main',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/core/timer',
    'com.voiceworks/ox-messenger/core/views/timer',
    'com.voiceworks/ox-messenger/conversation/main',
    'com.voiceworks/ox-messenger/ucconnect'

], function (ext, Stage, Capabilities, Session, Roster, Aggregator, TimerModel, TimerView, ConversationController, UCConnect) {

    /**
     * Badge View
     */
    var BadgeView = Backbone.View.extend({
        tagName: 'div',
        className: 'messenger--badge',

        events: {
            'click .messenger-js--badge-presence': 'showPresence',
            'click .messenger-js--badge-timer': 'showConversation',
            'click .messenger-js--badge-hangup': 'hangup',
            'click .messenger-js--badge-phone': 'togglePhone'
        },

        initialize: function () {
            var that = this;

            this.listenTo(this.model, 'change:status', this.render);
            this.listenTo(this.model.call, 'change:callState', this.render);
            this.listenTo(this.model.call, 'change:callSession', this.render);
            this.listenTo(this.model.call.timer, 'change:ticking', this.render);

            this.listenTo(Aggregator, 'sipSession:connected', this.render);
            this.listenTo(Aggregator, 'sipSession:disconnected', this.removePhone);

            this.$el.on('hover', function () {
                this.toggleClass('hover');
            }).attr({ href: '#', tabindex: '1' });
        },

        render: function () {
            var baton = ext.Baton({
                model: this.model
            });

            this.$el.empty();
            ext.point('io.ox/messenger/badge').invoke('draw', this.$el, baton);

            // It forces App Suite to re-position the Top Bar Views. Otherwise, badge might be rendered at the wrong place.
            $(window).trigger('resize');

            return this;
        },

        removePhone: function () {

            this.$el.find('.messenger-js--badge-phone').remove();
            // It forces App Suite to re-position the Top Bar Views. Otherwise, badge might be rendered at the wrong place.
            $(window).trigger('resize');
        },

        setNotifier: function (b) {
            this.$el.attr('aria-disabled', !b).find('.badge').toggleClass('active', !!b);
        },

        showPresence: function (e) {
            Aggregator.trigger('roster:toggle');

            if ($('.messenger--phone-overlay').length) {
                $('.messenger--phone-overlay').click();
            }
        },

        showConversation: function (e) {
            e.preventDefault();
            e.stopPropagation();

            if (!Session.user.call.isActive()) {
                return;
            }

            if (Session.user.call.get('type') === 'pstn') {
                Aggregator.trigger('phone:toggle');
            } else {
                ConversationController.showConversation(Session.user.conversations.get(this.model.call.get('conversationId')));
            }
        },

        hangup: function (e) {
            e.preventDefault();
            e.stopPropagation();

            if (this.model.call && Session.user.conversations) {
                var roomId = this.model.call.get('conversationId');
                var conversation = Session.user.conversations.get(roomId);

                Aggregator.trigger('call:hangup', {
                    conversation: conversation
                });
            }
        },

        togglePhone: function (e) {
            Aggregator.trigger('phone:toggle');

            if ($('.messenger--roster-overlay').length) {
                $('.messenger--roster-overlay').click();
            }
        }
    });

    ext.point('io.ox/messenger/badge').extend({
        id: 'presence-icon',
        index: 100,
        draw: function (baton) {
            if(baton.model && baton.model.call) {
                var badgeStatusClass = ' messenger--badge-presence  is-status--' + baton.model.get('status'),
                    badgeInACallClass = baton.model.call.isActive() ? ' is-status--in-a-call' : '';

                this.append(
                    $('<a class="messenger--badge-container  messenger-js--badge-presence">').append(
                        $('<span>').addClass('badge' + badgeStatusClass + badgeInACallClass).append(
                            $('<i>').addClass('fa fa-user')
                        )
                    )
                );
            }
        }
    });

    ext.point('io.ox/messenger/badge').extend({
        id: 'call-info',
        index: 200,
        draw: function (baton) {
            if(baton.model.call) {
                if (!baton.model.call.isActive()) {
                    return;
                }

                var timerView = new TimerView({
                        tagName: 'span',
                        model: baton.model.call.isActive() ? baton.model.call.timer : baton.conversation.timer
                    }),
                    timer = timerView.render().$el,
                    title = $('<span class="messenger--badge-timer__name">');

                if (baton.model.call.get('callSession')) {
                    switch (baton.model.call.get('type')) {
                        case 'pstn':
                            title.text(baton.model.call.get('callSession').display_name);
                            break;
                        case 'groupaudio':
                        case 'groupvideo':
                            title.append(
                                $('<i class="fa fa-group">'),
                                ' ',
                                baton.model.conversations.get(baton.model.call.get('conversationId')).get('title')
                            );
                            break;
                        default:
                            title.text(Roster.getUserById(baton.model.call.get('callSession').sip_uri).getDisplayName());
                    }
                } else {
                    title.text(baton.conversation.get('title'));
                }

                this.append(
                    $('<a class="messenger--badge-container  messenger-js--badge-timer">').append(
                        $('<span class="messenger--badge-timer  messenger--h5">').append(
                            title,
                            timer
                        )
                    )
                );
            }
        }
    });

    ext.point('io.ox/messenger/badge').extend({
        id: 'phone',
        index: 300,
        draw: function (baton) {
            if (UCConnect.capabilities.mustFlash || !Capabilities.has('messenger-phone') || baton.model.call.isActive()) {
                return;
            }

            var badgeStatusClass = ' messenger--badge-phone  is-status--' + baton.model.get('status'),
                badgeInACallClass = baton.model.call.isActive() ? ' is-status--in-a-call' : '';

            this.append(
                $('<a class="messenger--badge-container  messenger-js--badge-phone">').append(
                    $('<span>').addClass('badge' + badgeStatusClass + badgeInACallClass).append(
                        $('<i>').addClass('fa fa-phone')
                    )
                )
            );
        }
    });

    ext.point('io.ox/messenger/badge').extend({
        id: 'call-hangup',
        index: 300,
        draw: function (baton) {
            if (!baton.model.call.isActive()) {
                return;
            }

            this.append(
                $('<a class="messenger--badge-container  messenger-js--badge-hangup">').append(
                    $('<span>').addClass('badge  messenger--badge-hangup  is-status--xa').append(
                        $('<i>').addClass('fa fa-phone')
                    )
                )
            );
        }
    });

    var badgeView = new BadgeView({ model: Session.user });

    var stage = new Stage('io.ox/core/stages', {
        id: 'messenger-badge',
        before: 'load',
        run: function () {
            return require(['io.ox/core/main']).done(registerBadge);
        }
    });

    function registerBadge(coreMain) {
        // Add Messenger Badge to top bar
        ext.point('io.ox/core/topbar/right').extend({
            id: 'messenger',
            before: 'notifications',
            draw: function () {
                this.append(coreMain.addLauncher(
                    'right',
                    badgeView.render().$el,
                    function () {
                        //Aggregator.trigger('roster:toggle');
                }
                ).attr('id', 'messenger-launcher').addClass('messenger-launcher'));
            }
        });
    }
});
