define('com.voiceworks/ox-messenger/core/vendor/page-visible', [
], function () {
    'use strict';

    /**
     * Page Visibility with browser prefixes
     * source: http://davidwalsh.name/page-visibility
     */
    var PageVisible = {

        init: function () {
            var that = this;

            // Set name of hidden property and visibility change event
            // since some browsers only offer vendor-prefixed support
            if (typeof document.hidden !== 'undefined') {
                this.hidden = 'hidden';
                this.visibilityChange = 'visibilitychange';
                this.state = 'visibilityState';

            } else if (typeof document.mozHidden !== 'undefined') {
                this.hidden = 'mozHidden';
                this.visibilityChange = 'mozvisibilitychange';
                this.state = 'mozVisibilityState';

            } else if (typeof document.msHidden !== 'undefined') {
                this.hidden = 'msHidden';
                this.visibilityChange = 'msvisibilitychange';
                this.state = 'msVisibilityState';

            } else if (typeof document.webkitHidden !== 'undefined') {
                this.hidden = 'webkitHidden';
                this.visibilityChange = 'webkitvisibilitychange';
                this.state = 'webkitVisibilityState';
            }

            // Add a listener that constantly changes the title
            document.addEventListener(this.visibilityChange, function () {
                that.visibility = document[that.state];
            }, false);

            return this;
        },

        isVisible: function () {
            return this.visibility !== 'hidden';
        }
    };

    return PageVisible;
});