/**
 * Created by AdrianA on 5/11/2016.
 */
define('com.voiceworks/ox-messenger/core/views/ext/add-buddy', [
  'io.ox/participants/add',
  'io.ox/participants/views',
  'io.ox/core/notifications',
  'com.voiceworks/ox-messenger/core/session',
  'com.voiceworks/ox-messenger/roster/main',
  'io.ox/contacts/api',
  'gettext!com.voiceworks/ox-messenger'
], function (AddBuddiesView, pViews, notifications, Session, Roster, contactAPI, gt) {
  'use strict';

  /**
   * Add Buddy View
   * options.exclude: object with the email of the users to be excluded
   */

  var AddBuddy = AddBuddiesView.extend({
    constructor: function(options){
      var excludedUsers = _.extend(this.calculateExcludedUsers(), options.exclude),
        defaultOptions = {
          apiOptions: {
            contacts: true,
            users: true,
            groups: true,
            resources: true,
            distributionlists: true
          },
          className: '',
          blacklist: false, // blacklisted users are listed, but not selectable. So that we don't use it

          // Filter out the excluded users. Items are Raw OX contacts
          reduce: function(items){
            var filteredItems = items.filter(function(item){
                var keyEmail = item[item.field];

                var filtered = false;

                // item.field contains the name of the default email's field (email1,email2,email3)
                _.each(Session.user.contacts, function (contact) {
                  if (contact.email1 === keyEmail) {
                    filtered = true;
                  }
                });

                return (filtered && !excludedUsers.hasOwnProperty(keyEmail));
              });

            return filteredItems;
          },
          // Item is an autocomplete item
          suggestion: function(item) {
            var displayEmail    = item.get('email1') || item.get('email2'),
              keyEmail        = item.value,
              node            = $('<div class="autocomplete-item">');

            item.value = displayEmail;
            var user = Roster.getUserById(keyEmail);

            var imgDiv     = $('<div  class="messenger--media__img  media__img-avatar  messenger--avatar  messenger--avatar--small">').css('background-image', 'url(' + user.getImageUrl() + ')'),
              wrapperDiv = $('<div class="messenger--media__body">'),
              nameDiv    = $('<div class="messenger--h5  messenger--font--bold  messenger--buddy__username  messenger--text-wrap">').text(user.getDisplayName()).prepend(
                $('<span class="messenger--presence">').addClass('is-status--' + user.get('status'))
              ),
              emailDiv   = $('<div class="messenger--h5  messenger--buddy__status">').text(displayEmail);

            wrapperDiv.append(nameDiv, emailDiv);
            node.append(imgDiv, wrapperDiv);

            return node;


          }
        };

      _.defaults(options, defaultOptions);
      AddBuddiesView.apply(this, arguments);
    },
    addParticipant: function(e, model, value) {
      var excludedUsers = this.calculateExcludedUsers();

      if (typeof model.value === 'undefined') {
        for (var user in excludedUsers) {
          if (user === value) {
            notifications.yell('error', gt('"%s" is already a buddy.', value));
            return;
          }
        }
        for (var i = 0; i < this.collection.models.length; i++) {
          if (this.collection.models[i].id === value) {
            notifications.yell('error', gt('"%s" is already a buddy.', value));
            return;
          }
        }
      }

      // Check the selected user is in our buddy list
      if (!model.value) {

        var contact = Roster.getUserById(model.email1);

        if (contact === null) {
          model.id = model.email1;
          model.isNotContact = true;
        } else {
          var hasProperties = false;
          for (var key in contact) {
            if (Object.prototype.hasOwnProperty.call(contact, key))
              hasProperties = true;
          }
          if (!hasProperties) {
            model.id = model.email1;
            model.isNotContact = true;
          }

          if(!model.id){
            model.id = model.email1;
            model.isNotContact = true;
          }
        }

        if (!model.isNotContact) {
          // Extend the options with a modelParser function
          if (typeof this.options.modelParser === 'function') {
            model = this.options.modelParser(model);
          }
        }

        // To send invite when adding new buddy
        if (typeof this.options.inviteBuddy === 'function') {
          this.options.inviteBuddy(model, this);
        }

        var value = this.typeahead.$el.typeahead('val');
        AddBuddiesView.prototype.addParticipant.call(this, e, model, value);

        // OX api: Fix when value is "undefined"
        this.typeahead.$el.typeahead('val');

      }else{
        // Extend the options with a modelParser function
        if (typeof this.options.modelParser === 'function') {
          model = this.options.modelParser(model);
        }

        // To send invite when adding new buddy
        if (typeof this.options.inviteBuddy === 'function') {
          this.options.inviteBuddy(model, this);
        }

        var value = this.typeahead.$el.typeahead('val');
        AddBuddiesView.prototype.addParticipant.call(this, e, model, value);

        // OX api: Fix when value is "undefined"
        this.typeahead.$el.typeahead('val');
      }

    },
    calculateExcludedUsers: function(){
      var excludedUsers = {},
        buddies = Roster.getAllUsers();

      // Do not show yourself
      excludedUsers[Session.user.id] = true;

      // exclude already buddies
      _.each(Session.user.contacts, function (contact) {

        if (contact.email1) {
          if (buddies.get(contact.email1)) {
            excludedUsers[contact.email1] = true;
          }
        }
        if (contact.email2) {
          if (buddies.get(contact.email2)) {
            excludedUsers[contact.email2] = true;
          }
        }
        if (contact.email3) {
          if (buddies.get(contact.email3)) {
            excludedUsers[contact.email3] = true;
          }
        }
      });

      return excludedUsers;
    }
  });

  return AddBuddy;
});