/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/core/views/ext/add-participant', [
    'io.ox/participants/add',
    'io.ox/participants/views',
    'io.ox/core/notifications',
    'com.voiceworks/ox-messenger/core/session',
    'com.voiceworks/ox-messenger/roster/main',
    'gettext!com.voiceworks/ox-messenger'
], function (AddParticipantsView, pViews, notifications, Session, Roster, gt) {
    'use strict';

    /**
     * Add Participant View
     * options.exclude: object with the email of the users to be excluded
     */

    var AddParticipants = AddParticipantsView.extend({
        constructor: function(options){
            var excludedUsers = _.extend(this.calculateExcludedUsers(), options.exclude),
                defaultOptions = {
                    apiOptions: {
                        contacts: true,
                        users: true,
                        groups: true,
                        resources: true,
                        distributionlists: true
                    },
                    className: '',
                    blacklist: false, // blacklisted users are listed, but not selectable. So that we don't use it

                    // Filter out the excluded users. Items are Raw OX contacts
                    reduce: function(items){
                        var buddies = Roster.getAllUsers(),
                            filteredItems = items.filter(function(item){
                                var keyEmail = item[item.field];

                                // item.field contains the name of the default email's field (email1,email2,email3)
                                return (buddies.get(keyEmail) && !excludedUsers.hasOwnProperty(keyEmail));
                            });

                        return filteredItems;
                    },
                    // Item is an autocomplete item
                    suggestion: function(item) {
                        var buddies         = Roster.getAllUsers(),
                            displayEmail    = item.get('email1') || item.get('email2'),
                            keyEmail        = item.value,
                            user            = buddies.get(keyEmail),
                            node            = $('<div class="autocomplete-item">');

                        item.value = displayEmail;

                        var imgDiv     = $('<div  class="messenger--media__img  media__img-avatar  messenger--avatar  messenger--avatar--small">').css('background-image', 'url(' + user.getImageUrl() + ')'),
                            wrapperDiv = $('<div class="messenger--media__body">'),
                            nameDiv    = $('<div class="messenger--h5  messenger--font--bold  messenger--buddy__username  messenger--text-wrap">').text(user.getDisplayName()).prepend(
                                $('<span class="messenger--presence">').addClass('is-status--' + user.get('status'))
                            ),
                            emailDiv   = $('<div class="messenger--h5  messenger--buddy__status">').text(displayEmail);

                        wrapperDiv.append(nameDiv, emailDiv);
                        node.append(imgDiv, wrapperDiv);

                        return node;
                    }
                };

            _.defaults(options, defaultOptions);
            AddParticipantsView.apply(this, arguments);
        },
        addParticipant: function(e, model, value){
            var excludedUsers = this.calculateExcludedUsers();

            if(typeof model.value === 'undefined')
            {
                for ( var user in excludedUsers ) {
                    if(user === value){
                        notifications.yell('error', gt('"%s" is already part of the conversation.', value));
                        return;
                    }
                }
                for (var i=0; i< this.collection.models.length; i++ ) {
                    if(this.collection.models[i].id === value || this.collection.models[i].value === value){
                        notifications.yell('error', gt('"%s" is already part of the conversation.', value));
                        return;
                    }
                }
                notifications.yell('error', gt('"%s" is not a buddy. Please, select a user from your buddy list.', value));
                return;

            }

            // Check the selected user is in our buddy list
            if(!model.value)
            {
                notifications.yell('error', gt('"%s" is not a buddy. Please, select a user from your buddy list.', value));
                return;
            }

            // Extend the options with a modelParser function
            if (typeof this.options.modelParser === 'function') {
                model = this.options.modelParser(model);
            }

            // To send invite when adding new participants into already existing conversation
            if (typeof this.options.inviteBuddyInExistingConversation === 'function') {
                this.options.inviteBuddyInExistingConversation(model, this);
            }

            // Only add the participant when this flag is enabled (when creating a new MUC)
            if($(e.currentTarget).attr('call-add-participant')) {
                var value = this.typeahead.$el.typeahead('val');
                AddParticipantsView.prototype.addParticipant.call(this, e, model, value);
            }

            // OX api: Fix when value is "undefined"
            this.typeahead.$el.typeahead('val');
        },
        calculateExcludedUsers: function(){
            var excludedUsers = {},
                buddies = Roster.getAllUsers();

            // Do not show yourself
            excludedUsers[Session.user.id] = true;

            // only allow buddylist users
            _.each(Session.user.contacts, function (contact) {

                if (contact.email1) {
                    if (!buddies.get(contact.email1)) {
                        excludedUsers[contact.email1] = true;
                    }
                }
                if (contact.email2) {
                    if (!buddies.get(contact.email2)) {
                        excludedUsers[contact.email2] = true;
                    }
                }
                if (contact.email3) {
                    if (!buddies.get(contact.email3)) {
                        excludedUsers[contact.email3] = true;
                    }
                }
            });

            // Make sure that the budy has subscription to 'both' (can receive messages from me + send to me)
            buddies.each(function (user) {
                if (user.get('subscription') !== 'both') {
                    excludedUsers[user.id] = true;
                }
            });

            return excludedUsers;
        }
    });

    return AddParticipants;
});
