/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/desktop_notifications/main', [
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/core/vendor/notify',
    'com.voiceworks/ox-messenger/core/vendor/page-visible',
    'settings!com.voiceworks/ox-messenger'
], function (Aggregator, Notify, PageVisible, Settings) {
    'use strict';

    /**
     * Conversation Controller
     */
    var DesktopNotifications = {

        init: function (app) {
            Settings.on('change:showDesktopNotifications', this.register);

            this.register();

            return this;
        },

        register: function () {
            if (Settings.get('showDesktopNotifications')) {
                if (Notify.isSupported && Notify.needsPermission) {
                    Notify.requestPermission();
                }

                this.pageVisible = PageVisible.init();
            }
        },

        show: function (data) {
            if (!Settings.get('showDesktopNotifications') || !Notify.isSupported || (this.pageVisible && this.pageVisible.isVisible())) {
                return;
            }

            this.notification = new Notify(data.title, {
                body: data.message,
                tag: data.tag,
                icon: data.icon,
                notifyClick: data.callback,
                timeout: Settings.get('desktopNotificationTimeout')
            });

            this.notification.show();
        }
    };

    return DesktopNotifications;
});
