/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/history/main', [
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/core/session',
    'com.voiceworks/ox-messenger/history/models/historycollection',
    'com.voiceworks/ox-messenger/history/models/history',
    'com.voiceworks/ox-messenger/history/views/history',
    'com.voiceworks/ox-messenger/history/views/list',
    'settings!com.voiceworks/ox-messenger'
], function (Aggregator, Session, HistoryCollection, HistoryModel, HistoryView, HistoryListView, settings) {
    'use strict';

    var CallHistory = {
        fetch: function (limit) {
            var that = this;

            $.ajax({
                url: settings.get('API_LOCATION'),
                dataType: 'jsonp',
                data: {
                    method: 'client.getcdrs',
                    params: {
                        sessionId: ox.session,
                        targetDomain: '',
                        targetUser: '',
                        offset: 0,
                        limit: limit,
                        mode: ''
                    }
                }
            }).done(function (data) {
                var models = [];

                _.each(data.response.cdrs, function (historyItem) {
                    models.push(new HistoryModel(historyItem));
                });

                Session.user.callHistory.reset(models);

                Aggregator.trigger('callhistory:received');
            });
        },

        show: function () {
            // Hide previous history screen
            var that = this;
            this.hide();

            Aggregator.on('history:close', $.proxy(this.hide, this));

            require(['com.voiceworks/ox-messenger/main'], function (main) {
                var listView = new HistoryListView({
                    app: main.getApp(),
                    ignoreFocus: false,
                    pagination: false
                });

                that.historyView = new HistoryView({
                    listView: listView
                });

                listView.setCollection(Session.user.callHistory);

                listView.on('selection:change', function (ids) {
                    //console.info('SELECT handler', ids);
                });
                listView.collection.trigger('reset');

                // Hide checkboxes in history list view
                listView.toggleCheckboxes(false);

                main.getApp().launch().then(function () {
                    main.getApp().right.append(
                        that.historyView.render().$el
                    );
                });
            });
        },

        hide: function () {
            require(['com.voiceworks/ox-messenger/main'], function (main) {
                if(main.getApp().right) {
                    main.getApp().launch().then(function () {
                        main.getApp().right.find('#messenger--call-history').remove();
                    });
                }
            });

            if (this.historyView && this.historyView.$el) {
                this.historyView.remove();
            }
        }
    };

    return CallHistory;
});
