/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/history/views/list', [
    'io.ox/core/extensions',
    'io.ox/core/tk/list',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/roster/main',
    'com.voiceworks/ox-messenger/core/contactsearch',
    'com.voiceworks/ox-messenger/util',
    'gettext!com.voiceworks/ox-messenger',
    'less!com.voiceworks/ox-messenger/history/style'
], function (ext, ListView, Aggregator, Roster, ContactSearch, Util, gt) {
    'use strict';

    ext.point('com.voiceworks/ox-messenger/history/list-view/item').extend({
        id: 'default',
        draw: function (baton) {
            var tableElement = $('<div class="messenger--table  messenger--table--cdr">');
            var rowElement = $('<div class="messenger--table__row">');
            ext.point('com.voiceworks/ox-messenger/history/list-view/item/default').invoke('draw', rowElement, baton);
            this.append(tableElement.append(rowElement));

            this.parent().on('click', function (event) {
                if (event.target.nodeName === 'A') {
                    return;
                }
                event.preventDefault();
                event.stopPropagation();

                $(event.target).find('a:first').trigger('click');
            });
        }
    });

    ext.point('com.voiceworks/ox-messenger/history/list-view/item/default').extend({
        id: 'call-type',
        index: 100,
        draw: function (baton) {
            var type = $('<div class="messenger--table__cell  messenger--table__cell--cdr  messenger--table__cell--cdr--icon">');

            if (baton.model.isPstn()) {
                type.append($('<i class="fa fa-phone">'));
            } else {
                type.append($('<i class="fa fa-desktop">'));
            }

            this.append(type);
        }
    });

    ext.point('com.voiceworks/ox-messenger/history/list-view/item/default').extend({
        id: 'call-party',
        index: 200,
        draw: function (baton) {
            var party = $('<div class="messenger--table__cell  messenger--table__cell--cdr  messenger--table__cell--cdr--name">'),
                otherUser = baton.model.getOtherParty();

            if (baton.model.isPstn()) {
                var contacts = ContactSearch.byPhonenumber(otherUser),
                    link = $('<a href="#">').on('click', function (e) {
                        e.preventDefault();
                        Aggregator.trigger('phone:toggle', { phoneNumber: otherUser });
                    });

                if (contacts[0]) {
                    link.append(
                        $('<span>').text(contacts[0].name)
                    );
                    // Display name + number type
                    party.append(
                        link,
                        $('<span class="tmp-mes-numbertype">').text(' (' + contacts[0].numberType + ') ')
                    );
                } else {
                    // Display number
                    party.append(link.text(otherUser));
                }
            } else {
                party.append($('<a href="#" class="halo-link">').data({ email1: otherUser }).text(Roster.getUserById(otherUser).getDisplayName()));
            }

            this.append(party);
        }
    });

    ext.point('com.voiceworks/ox-messenger/history/list-view/item/default').extend({
        id: 'call-duration',
        index: 300,
        draw: function (baton) {
            var duration = $('<div class="messenger--table__cell  messenger--table__cell--cdr  messenger--table__cell--cdr--duration">'),
                durationText = $('<span>'),
                durationSeconds = Math.round(parseInt(baton.model.get('billmsec'), 10) / 1000),
                durationDisplay = Util.getDurationFormatted(durationSeconds),
                hangupCause = baton.model.getCallHangupCause(),
                failed = false;

            switch (hangupCause) {
            case 'cancelled':
                failed = gt('Call canceled');
                break;
            case 'missed':
                failed = gt('Call missed');
                break;
            }

            if (failed) {
                durationText.text(failed).addClass('messenger--text--missed-call');
            } else {
                duration.text(durationDisplay);
            }

            this.append(duration.append(durationText));
        }
    });

    ext.point('com.voiceworks/ox-messenger/history/list-view/item/default').extend({
        id: 'call-startdate',
        index: 400,
        draw: function (baton) {
            var jsDate = new Date(baton.model.getStartStamp()),
                startDate = Util.formatDateTime(jsDate.getTime());

            this.append(
                $('<div class="messenger--table__cell  messenger--table__cell--cdr  messenger--table__cell--cdr--date">').append(
                    $('<time class="date">')
                    .attr('datetime', Util.getDateYmdHms(jsDate))
                    .text(_.noI18n(startDate))
                )
            );
        }
    });

    var HistoryListView = ListView.extend({
        ref: 'com.voiceworks/ox-messenger/history/list-view',

        initialize: function (options) {
            ListView.prototype.initialize.call(this, options || {});
            this.$el.addClass('history-item');
        },

        setCollection: function (collection) {
            ListView.prototype.setCollection.call(this, collection);
            this.listenTo(this.collection, 'add change reset', this.render);
        },

        map: function (model) {
            return model.toJSON();
        },

        // support for custom cid attributes needed to identify threads
        getCID: function (model) {
            return model.id;
        }
    });

    return HistoryListView;
});