/**
/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

/* global Strophe */

define('com.voiceworks/ox-messenger/mediator', [
    'io.ox/contacts/api',
    'io.ox/core/notifications',
    'io.ox/core/capabilities',
    'io.ox/core/folder/api',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/ucconnect',
    'com.voiceworks/ox-messenger/core/session',
    'com.voiceworks/ox-messenger/core/notification-control',
    'com.voiceworks/ox-messenger/core/notification',
    'com.voiceworks/ox-messenger/roster/main',
    'com.voiceworks/ox-messenger/phone/main',
    'com.voiceworks/ox-messenger/conversation/main',
    'com.voiceworks/ox-messenger/conversation/models/conversation',
    'com.voiceworks/ox-messenger/conversation/models/groupconversation',
    'com.voiceworks/ox-messenger/conversation/models/message/factory',
    'com.voiceworks/ox-messenger/conversation/models/message/system',
    'com.voiceworks/ox-messenger/conversation/models/messagearchive',
    'com.voiceworks/ox-messenger/util',
    'com.voiceworks/ox-messenger/desktop_notifications/main',
    'com.voiceworks/ox-messenger/history/main',
    'com.voiceworks/ox-messenger/history/models/history',
    'com.voiceworks/ox-messenger/core/views/partials/userpart',
    'com.voiceworks/ox-messenger/file/main',
    'gettext!com.voiceworks/ox-messenger',
    'settings!com.voiceworks/ox-messenger',
    'settings!io.ox/core'
], function (ContactsApi, CoreNotification, Capabilities, folderApi, Aggregator, UCConnect,
                         Session, NotificationControl, Notification, Roster, Phone, Conversation, ConversationModel,
                         GroupConversationModel, MessageFactory, SystemMessageModel, MessageArchiveModel, Util, DesktopNotifications,
                         CallHistory, HistoryModel, UserPart, FileController, gt, Settings, CoreSettings) {
    'use strict';

    /**
     * Enum with allowed call types.
     *
     * TODO move to a separate namespace.
     */
    var CALL_TYPE = {
        video: true,
        audio: true,
        groupvideo: true,
        groupaudio: true
    };

    var _currentAppName;
    var _conversationsStorage = {};

    function assert(condition, msg) {
        if (!condition) {
            throw new Error(msg || '');
        }
    }

    var Mediator = {
        init: function () {
            var that = this;

            this.desktopNotification = DesktopNotifications.init();
            this.msgArchives = [];
            this.msgArchiveCount = [];

            var initialPresence = Settings.get('initialPresenceStatus');
            if (!initialPresence) {
                initialPresence = 'online';
            }

            UCConnect.init({
                initialPresence: initialPresence,
                logEnabled: _.url.hash('debug-messenger')
                //logConnector: $.proxy(Session.logger.write, this)
            });

            UCConnect.events.on('call_state', $.proxy(this.callStateHandler, this));
            UCConnect.events.on('connecting', $.proxy(this.connectingHandler, this));
            UCConnect.events.on('connected', $.proxy(this.connectedHandler, this));
            UCConnect.events.on('disconnected', $.proxy(this.disconnectedHandler, this));
            UCConnect.events.on('end_call', $.proxy(this.endCallHandler, this));
            UCConnect.events.on('failed', $.proxy(this.failedHandler, this));
            UCConnect.events.on('invite', $.proxy(this.groupInvitedHandler, this));
            UCConnect.events.on('local_media_requested', $.proxy(this.localMediaRequestedHandler, this));
            UCConnect.events.on('local_media_failed', $.proxy(this.localMediaFailedHandler, this));
            UCConnect.events.on('local_media_acquired', $.proxy(this.localMediaAcquiredHandler, this));
            UCConnect.events.on('rsm', $.proxy(this.rsmHandler, this));
            UCConnect.events.on('message', $.proxy(this.messageHandler, this));
            UCConnect.events.on('memberlist', $.proxy(this.memberListHandler, this));
            UCConnect.events.on('mystatus', $.proxy(this.myStatusHandler, this));
            UCConnect.events.on('new_call', $.proxy(this.newCallHandler, this));
            UCConnect.events.on('parameters', $.proxy(this.parametersHandler, this));
            UCConnect.events.on('mucs', $.proxy(this.mucsHandler, this));
            UCConnect.events.on('status', $.proxy(this.statusHandler, this));
            UCConnect.events.on('vcard_updated', $.proxy(this.vcardUpdatedHandler, this));
            UCConnect.events.on('chat_state', $.proxy(this.chatStateHandler, this));
            UCConnect.events.on('subscribe', $.proxy(this.subscribeHandler, this));
            UCConnect.events.on('archivecollection', $.proxy(this.archiveCollectionHandler, this));
            UCConnect.events.on('conversationdeleted', $.proxy(this.conversationDeletedHandler, this));
            UCConnect.events.on('groupcallinvite', $.proxy(this.groupCallInviteHandler, this));
            UCConnect.events.on('groupcallready', $.proxy(this.groupCallReadyHandler, this));
            UCConnect.events.on('groupcallcancelled', $.proxy(this.groupCallCancelledHandler, this));
            UCConnect.events.on('groupcalldeclined', $.proxy(this.groupCallDeclinedHandler, this));
            UCConnect.events.on('groupcallstatus', $.proxy(this.groupCallStatusHandler, this));
            UCConnect.events.on('roster', $.proxy(this.rosterHandler, this));

            Aggregator.on('conversation:delete', $.proxy(this.deleteConversationHandler, this));
            Aggregator.on('conversation:show', $.proxy(this.conversationStartHandler, this));
            Aggregator.on('conversation:state', $.proxy(this.conversationStateHandler, this));

            Aggregator.on('call:init', $.proxy(this.conversationStartHandler, this));
            Aggregator.on('call:accept', $.proxy(this.callAcceptHandler, this));
            Aggregator.on('call:decline', $.proxy(this.callDeclineHandler, this));
            Aggregator.on('call:hangup', $.proxy(this.callHangupHandler, this));
            Aggregator.on('call:unsuccessful', $.proxy(this.callUnsuccessfulHandler, this));

            Aggregator.on('call:issueDetected', $.proxy(this.callIssueDetectedHandler, this));

            Aggregator.on('phone:call', $.proxy(this.phoneCallHandler, this));
            Aggregator.on('phone:hangup', $.proxy(this.phoneHangupHandler, this));
            Aggregator.on('phone:hold', $.proxy(this.phoneHoldHandler, this));
            Aggregator.on('phone:dtmf', $.proxy(this.phoneDtmfHandler, this));

            Aggregator.on('groupcall:init', $.proxy(this.groupCallStartHandler, this));
            //Aggregator.on('groupcall:ready', $.proxy(this.groupCallReadyHandler, this));
            //Aggregator.on('groupcall:cancel', $.proxy(this.groupCallCancelHandler, this));
            Aggregator.on('groupcall:accept', $.proxy(this.groupCallAcceptHandler, this));
            Aggregator.on('groupcall:rejoin', $.proxy(this.groupCallAcceptHandler, this));
            Aggregator.on('groupcall:decline', $.proxy(this.groupCallDeclineHandler, this));
            Aggregator.on('groupcall:sendInvite', $.proxy(this.groupCallSendInviteHandler, this));

            Aggregator.on('group:leave', $.proxy(this.groupLeaveHandler, this));
            Aggregator.on('group:join', $.proxy(this.groupJoinHandler, this));
            Aggregator.on('group:create', $.proxy(this.groupCreateHandler, this));
            Aggregator.on('group:invite', $.proxy(this.groupInviteHandler, this));
            Aggregator.on('group:subject', $.proxy(this.groupSubjectHandler, this));
            Aggregator.on('group:createNew', $.proxy(this.groupCreateNewHandler, this));
            Aggregator.on('group:createNewCall', $.proxy(this.groupCreateNewCallHandler, this));

            Aggregator.on('groupconversation:changed', $.proxy(this.groupConversationChangedHandler, this));
            Aggregator.on('groupconversation:next-rsm', $.proxy(this.groupConversationNextPageHandler, this));
            Aggregator.on('groupconversation:name', $.proxy(this.groupConversationNameHandler, this));
            Aggregator.on('groupconversation:subject', $.proxy(this.groupConversationSubjectHandler, this));

            Aggregator.on('session:status', $.proxy(this.sessionStatusHandler, this));

            Aggregator.on('session:statusManuallyChanged', $.proxy(this.statusManuallyChangedHandler, this));

            Aggregator.on('session:setup', $.proxy(this.sessionSetupHandler, this));
            Aggregator.on('session:reloadContacts', $.proxy(this.sessionReloadContactsHandler, this));

            Aggregator.on('roster:add-guest', $.proxy(this.rosterAddGuestHandler, this));
            Aggregator.on('roster:remove-guest', $.proxy(this.rosterRemoveGuestHandler, this));
            Aggregator.on('roster:remove-buddy', $.proxy(this.rosterRemoveBuddyHandler, this));
            Aggregator.on('roster:group:changed', $.proxy(this.rosterGroupChangedHandler, this));
            Aggregator.on('roster:group:create', $.proxy(this.rosterGroupCreateHandler, this));
            Aggregator.on('roster:group:update', $.proxy(this.rosterGroupUpdateHandler, this));
            Aggregator.on('roster:group:delete', $.proxy(this.rosterGroupDeletedHandler, this));
            Aggregator.on('roster:group:toggle', $.proxy(this.rosterGroupToggledHandler, this));

            Aggregator.on('message:send', $.proxy(this.sendMessageHandler, this));

            Aggregator.on('history:refresh', $.proxy(this.refreshCallHistoryHandler, this));

            Aggregator.on('callhistory:received', $.proxy(this.callHistoryReceivedHandler, this));
            Aggregator.on('xmpp-history:loaded', $.proxy(this.fetchOfflineMessages, this));

            // Settings
            Settings.on('change:showChatNotifications', this.handleSettingsChatNotifications);
            Aggregator.on('conversation:showNotifications', $.proxy(this.handleConversationNotifications, this));
            // Aggregator.on('xmpp-history:loaded', $.proxy(this.setupConversationNotificationSettings, this));
            // Aggregator.on('muc-history:loaded', $.proxy(this.setupConversationNotificationSettings, this));
            Aggregator.on('conversation:setupNotifications', $.proxy(this.setupConversationNotificationSettings, this));
            //Aggregator.on('scroll-up', $.proxy(this.testHandler, this));

            Aggregator.on('conversation:reloadCurent', $.proxy(this.handleReloadCurentConversation, this));
            Aggregator.on('focus:textarea', $.proxy(this.handleFocusTextArea, this));

            Aggregator.on('muc:tryCallAgain', $.proxy(this.handleTryCallAgain, this));

            Aggregator.on('sipSession:disconnected', $.proxy(this.handleSipSessionDisconnected, this));
            Aggregator.on('sipSession:connected', $.proxy(this.handleSipSessionConnected, this));
            // OX
            ox.on('app:init', $.proxy(this.onAppChange, this));
            ox.on('app:resume', $.proxy(this.onAppChange, this));
            ox.on('connection:offline', $.proxy(this.offlineHandler, this));
            //ox.on('relogin:success', $.proxy(this.reloginHandler, this));

            Session.setup();

            Roster.attach();

            if (Capabilities.has('messenger-phone')) {
                Phone.attach();
            }

            // Listen to contact data updates
            ContactsApi.on('update', function(event, data) {
                // Check if we updated ourselves
                if (Session.user.get('contact').id == data.user_id) {
                    var imgUrl = data.image1_url || '';

                    Session.user.setContact(data);

                        // Notify contacts about the new profile picture
                    UCConnect.vcard(imgUrl, data.first_name, data.last_name);
                    UCConnect.status(Session.user.get('status'), Session.user.get('statusMessage'));
                }
            });

            // Check computer sleep and refresh session if necessary
            Util.startWakeup(onWakeup);
            function onWakeup(sleepTime, invtervalId){
                that.log('I have slept for ' + sleepTime + ' seconds');
                if(Session.get('connection') === 'offline'){
                    that.log('Refreshing session');
                    Session.setup();
                }

                Util.stopWakeup(invtervalId);
                Util.startWakeup(onWakeup);
            }

            //OXM-1171 - Emoticons are displayed differently
            Util.processChatText(':)');

            window.addEventListener('unload', function (event) {
                UCConnect.disconnect();
            });

            $(window).on('roster:groups:reset', $.proxy(this.rosterGroupResetHandler, this));

            FileController.patchOXFileDeepLinks();

            /**
             * This is just for debugging perposes
             */
                window.Aggregator = Aggregator;
        },

        handleSipSessionDisconnected: function () {
            this.log('handleSipSessionDisconnected');

            Session.user.set('canAudio', false);
            Session.user.set('canVideo', false);
            UCConnect.getRoster('xmpp');
        },

        handleSipSessionConnected: function () {

            this.log('handleSipSessionConnected');

            Session.user.set('canAudio', true);
            Session.user.set('canVideo', true);
            UCConnect.getRoster('xmpp');
        },

        handleFocusTextArea: function() {
            if (Conversation.conversationView) {
                Conversation.conversationView.messageFormView.focusInput();
            }
        },

        /*
         * Messenger event handlers
         * ================================================
         */
        sessionSetupHandler: function (data) {
            this.log('sessionSetupHandler', data);

            // Set XMPP nickname to allow guest users to see OX user's full name
            UCConnect.setNickname(Session.user.getDisplayName());

            //@TODO: handle result
            var result = UCConnect.connect(data);

            Roster.init(data.userObj);

            if (Capabilities.has('messenger-phone')) {
                Phone.init(data.userObj);
            }
        },

        sessionStatusHandler: function (data) {
            this.log('sessionStatusHandler', data);

            if (this.isOnline()) {

                if (data.status.toLowerCase() !== Session.user.get('status').toLowerCase()) {
                    var currentCallConversation = Session.user.getCurrentCallConversation();
                    Session.user.setStatus(data.status, data.statusMessage);
                    UCConnect.status(data.status, data.statusMessage);
                    if (currentCallConversation) {
                        // Update the user status in the current call conversation/room
                        UCConnect.groupCallStatus(currentCallConversation.get('roomHost'), currentCallConversation.get('id'),
                                data.statusMessage);
                    }
                }
            }
            $('.messenger--presence[data-jid="' + Session.user.get('id') + '"]')
                    .removeClass('is-status--unknown ' +
                            'is-status--offline ' +
                            'is-status--online ' +
                            'is-status--away ' +
                            'is-status--dnd ' +
                            'is-status--invisible ' +
                            'is-status--xa')
                    .addClass('is-status--' + data.status);

            // Store conversations' current localStorage to the privateXML
            $('a[data-action="logout"]').on('click', function(){
                var conversationsStorage = {};

                // Fill the array with the current
                Session.user.conversations.each(function (conversation) {
                    conversationsStorage[conversation.get('id')] = conversation.getLocalStorage();
                    });

                UCConnect.storePrivateXml('xmpp', 'conversations:localstorage', JSON.stringify(conversationsStorage));
            });
        },

        statusManuallyChangedHandler: function (data) {
                this.log('statusManuallyChangedHandler', data);
                Session.user.set('prev_status', data.status.toLowerCase());
        },

        sessionReloadContactsHandler: function () {
            this.log('sessionReloadContactsHandler');

            Session.reloadContacts(Session);
        },

        archiveCollectionHandler: function (event) {
            this.log('archiveCollectionHandler', event);

            var archives = event.data.archive;

            if (event.data.archive.length < 1) {
                Session.set('xmppHistoryLoaded', true);
                Aggregator.trigger('xmpp-history:loaded');
            }

            for (var i = 0; i < archives.length; i++) {
                 var bareJid    = Util.getBareJid(archives[i].jid);
                 var resource = Util.getResourceFromJid(archives[i].jid);
                 if (Session.user.conversations.get(bareJid) &&
                    Session.user.conversations.get(bareJid).isMarkedForDeletion()) {
                    Conversation.delete(archives[i].jid);
                    UCConnect.deleteConversation('xmpp', archives[i].jid, archives[i].startDate);
                 }
                 else {
                    var conversation = this.startConversation(bareJid, 'chat');
                    var archiveModel = new MessageArchiveModel({
                             archive: archives[i],
                             user: Roster.getUserById(bareJid),
                             jid: bareJid,
                             resource: resource,
                             startDate: archives[i].startDate
                     });
                     conversation.get('jidToDate').push({
                             jid: archives[i].jid,
                             date: archives[i].startDate
                     });
                     // TODO: should be handled by the setter
                     if (!conversation.get('startDate') || archives[i].startDate < conversation.get('startDate')) {
                             conversation.set('startDate', archives[i].startDate);
                     }

                     conversation.set('timestamp', new Date(archives[i].startDate).getTime());

                     conversation.messageArchives.add(archiveModel);
                     // TODO potential double sort
                     //conversation.messageArchives.sort();
                 }
            }

            var leftForDeletion = [];

            Session.user.conversations.each(function (conversation) {
                if (conversation.isMarkedForDeletion()) {
                    leftForDeletion.push(conversation.get('id'));
                }
                else {
                    if (!conversation.isMuc()) {
                       conversation.getNextArchiveMessage();
                    }
                }
            });

            while (leftForDeletion.length > 0) {
                Conversation.delete(leftForDeletion.shift());
            }

            // TODO execute once all the messageCountCb are finished
            Session.set('xmppHistoryLoaded', true);
            Aggregator.trigger('xmpp-history:loaded');
            this.sortConversations(null);
        },

        callHistoryReceivedHandler: function (data) {
            this.log('callHistoryReceivedHandler', data);

            var fetchAllConersations = function() {
                // Fetch XMPP history for 1:1 conversations
                UCConnect.fetchHistory('xmpp', '', null);

                // Fetch XMPP history for MUCs
                UCConnect.fetchMucs('xmpp', Settings.get('groupConversationRoomHost'));
            };

            // Get the conversations localstorage values (stored when logged out)
            UCConnect.fetchPrivateXml('xmpp', 'conversations:localstorage', function (data) {
                try{
                    _conversationsStorage = JSON.parse(data);
                }catch(e){}

                fetchAllConersations();
            });
        },

        fetchOfflineMessages: function () {
            this.log('fetchOfflineMessages');

            UCConnect.getOfflineMessages('xmpp', true);
        },

        deleteConversationHandler: function () {
            this.log('deleteConversationHandler');

            if (!Conversation.listView.selection) {
                return;
            }

            var selectedItems = Conversation.listView.selection.get(),
                mucChanged = false,
                prevCallConversation,
                currentCallConversation = Session.user.getCurrentCallConversation(),
                needToFetchHistory = false;

            _.each(selectedItems, function (selectedItem) {
                var selectedConversation = Session.user.conversations.get(selectedItem);

                if (currentCallConversation && selectedConversation.get('id') === currentCallConversation.get('id')) {
                    prevCallConversation = selectedConversation;
                    CoreNotification.yell('warning', gt('The conversation with an active call was not deleted.'));
                    Session.user.conversations.currentConversationId = null;
                    return;
                }

                if (Session.user.conversations.currentConversationId === selectedConversation.id) {
                    Session.user.conversations.currentConversationId = null;
                }

                if (selectedConversation.isMuc()) {
                    UCConnect.groupLeave('xmpp', selectedConversation.get('roomHost'), selectedConversation.get('roomId'));
                    Session.user.conversations.remove(selectedConversation);
                    mucChanged = true;
                } else {
                    // TODO remove all the jids present in the conversation
                    var jidToDate = selectedConversation.get('jidToDate');

                    if (Util.isEmpty(jidToDate)) {
                        jidToDate.forDelete = true;
                        needToFetchHistory = true;
                        selectedConversation.set('jidToDate', jidToDate);
                    }
                    else {
                        Conversation.delete(selectedItem);
                        jidToDate.forEach(function (item) {
                            console.debug('DBG', selectedItem);
                            UCConnect.deleteConversation('xmpp', item.jid, item.date);
                        });
                    }
                }
            });

            if (needToFetchHistory) {
                UCConnect.fetchHistory('xmpp');
            }

            if (mucChanged) {
                Session.user.conversations.trigger('reset');
                Conversation.conversationDeleted();
                Aggregator.trigger('groupconversation:changed', {});
            }

            if (prevCallConversation) {
                Conversation.listView.selection.set([prevCallConversation.cid]);
            } else {
                var firstConversation = Session.user.conversations.first();
                if (firstConversation) {
                    Conversation.listView.selection.set([firstConversation.cid]);
                }
            }
            Aggregator.trigger('conversationlist:selection:change', []);
        },

        conversationDeletedHandler: function (data) {
            this.log('conversationDeletedHandler', data);

        },

        groupConversationChangedHandler: function (data) {
            this.log('groupConversationChangedHandler', data);

            var mucRooms = [];

            _.each(Session.user.conversations.fetchMucRooms(), function (conversation) {
                var participants = conversation.participants.pluck('id'),
                        isMember = conversation.participants.get(Session.user) ? true : false;

                mucRooms.push({
                    roomId: conversation.get('roomId'),
                    roomHost: conversation.get('roomHost'),
                    roomName: Util.b64EncodeUnicode(conversation.get('roomName')),
                    subject: Util.b64EncodeUnicode(conversation.get('subject')),
                    myRoom: conversation.get('myRoom'),
                    participants: participants,
                    isMember: isMember,
                    timestamp: conversation.get('timestamp')
                });
            });
        },

        groupConversationNameHandler :function (data) {
            var self = this;
            var id = data.id;

            this.log('groupConversationNameHandler', data);

            var conversation = data.conversation;

            UCConnect.groupEdit('xmpp', conversation.get('roomHost'), conversation.get('roomId'), data.name);
            this.sortConversations(null);
            return;
        },

        groupConversationSubjectHandler :function (data) {
            this.log('groupConversationSubjectHandler', data);

            var conversation = data.conversation;

            UCConnect.groupSubject('xmpp', conversation.get('roomHost'), conversation.get('roomId'), data.subject);
            this.sortConversations(null);
            return;
        },

        groupConversationNextPageHandler: function(data) {
                UCConnect.fetchMucRsm('xmpp', data.conversationId, data.firstMsgDbId, data.limit);
        },

        memberListHandler: function (event) {
            var conversation = Session.user.conversations.get(event.data.room);
            if (conversation) {
                var previousParticipantList = [],
                        currentParticipantsList = event.data.memberList,
                        participantsLeft,
                        participantsJoined,
                        firstLoad = conversation.participants.length === 0,
                        user;

                conversation.participants.each(function (participant) {
                    previousParticipantList.push(participant.id);
                });
                _.uniq(previousParticipantList);

                participantsLeft = _.difference(previousParticipantList, currentParticipantsList);
                participantsJoined = _.difference(currentParticipantsList, previousParticipantList);

                _.each(participantsLeft, function (participant) {
                    user = Roster.getUserById(participant);
                    conversation.removeParticipant(user);
                    if (Session.user.id === participant) {
                        conversation.properties.set('groupMember', false);
                    }
                });
                _.each(participantsJoined, function (participant) {
                    user = Roster.getUserById(participant);
                    // If it's firstLoad, we don't create system messages
                    conversation.addParticipant(user, false, firstLoad);
                    if (Session.user.id === participant) {
                        conversation.properties.set('groupMember', true);
                    }
                });
                Aggregator.trigger('groupconversation:changed', {});
            }
        },

        refreshCallHistoryHandler: function () {
            this.log('refreshCallHistoryHandler');

            Phone.view.historyView.getCdrs();
        },

        rosterAddGuestHandler: function (data) {
            this.log('rosterAddGuestHandler', data);

            var that = this;

            // Fetch roster with provisioned guest
            UCConnect.getRoster('xmpp');

            Aggregator.on('roster:created', function () {
                // TODO unbind only this listener, no all of them
                Aggregator.off('roster:created');

                var user = Roster.getUserById(data.username);

                // user.properties.addProperty('Guest Users', true);
                user.properties.addProperty(3, true); // 3 = Guest Users

                UCConnect.rosterUserGroupsSet('xmpp', user.get('id'), user.properties.toArray());

                // (Temporarily) store the guest user in the roster with the chosen nickname and e-mail address
                user.set({
                    nickname: _.escape(data.name),
                    contact: {
                        email1: data.email1
                    }
                });
            });
        },

        rosterRemoveGuestHandler: function (data) {

            var that = this,
                    question;

            question = gt('Do you really want to delete this guest-user?');

            require([
                'io.ox/core/tk/dialogs',
                'io.ox/core/http',
                'io.ox/contacts/api',
                'io.ox/core/notifications'
            ], function (dialogs, http, ContactsApi, notifications) {
                new dialogs.ModalDialog()
                        .text(question)
                        .addPrimaryButton('delete', gt('Delete'), 'delete', {'tabIndex': '1'})
                        .addButton('cancel', gt('Cancel'), 'cancel', {'tabIndex': '1'})
                        .show()
                        .done(function (action) {
                            if (action === 'delete') {
                                Mediator.removeGuestData(data.user.id);
                                http.GET({
                                    module: 'messenger',
                                    params: {
                                        action: 'delete_guest_user',
                                        session: ox.session,
                                        mail: data.user.id
                                    }
                                }).done(function (result) {
                                            ContactsApi.getByEmailaddress(data.user.id).done(function (contact) {
                                                ContactsApi.remove({
                                                    folder_id: contact.folder_id,
                                                    id: contact.id
                                                }).done(function (contact) { })
                                                        .fail(function (e) {
                                                            notifications.yell('error', gt('Error deleting guest account contact for %s', data.user.id));
                                                            console.error(e);
                                                        });
                                            });
                                        })
                                        .fail(function (e) {
                                            notifications.yell('error', gt('Error deleting guest account for %s', data.user.id));
                                        });
                            }
                        });
            });
        },

        rosterRemoveBuddyHandler: function (data) {

            var that = this,
                    question;

            question = gt('Do you really want to delete this buddy?');

            require([
                'io.ox/core/tk/dialogs',
                'io.ox/core/http',
                'io.ox/contacts/api',
                'io.ox/core/notifications'
            ], function (dialogs, http, ContactsApi, notifications) {
                new dialogs.ModalDialog()
                        .text(question)
                        .addPrimaryButton('delete', gt('Delete'), 'delete', {'tabIndex': '1'})
                        .addButton('cancel', gt('Cancel'), 'cancel', {'tabIndex': '1'})
                        .show()
                        .done(function (action) {
                            if (action === 'delete') {
                                Mediator.removeGuestData(data.user.id);
                            }
                        });
            });
        },
        // Triggered when the properties model changes
        // @TODO: loop???
        rosterGroupChangedHandler: function (object) {
            this.log('rosterGroupChangedHandler', object);

            UCConnect.rosterUserGroupsSet('xmpp', object.user.get('id'), object.values);
        },

        handleFetchRosterGroups: function (data) {
            this.log('handleFetchRosterGroups', data);

            var storedGroups;

            // Check if the returned data is a valid JSON object
            try{
                storedGroups = JSON.parse(data);
            }catch(e){
                return;
            }

            // Find missing groups between PrivateXML and Roster
            var rosterGroupIds = Roster.getGroups().models.map(function (group) { return parseInt(group.id, 10); }),
                    storedGroupIds = _.keys(storedGroups).map(function (group) { return parseInt(group, 10); }),
                    missingGroups    = _.difference(storedGroupIds, rosterGroupIds);

            // Add missing groups
            _.each(missingGroups, function (groupId) {
                Roster.createGroup(storedGroups[groupId], parseInt(groupId, 10));
            });

            // Set names from PrivateXML for all groups
            var groups = Roster.getGroups();
            groups.models.forEach(function(group) {
                if (typeof group.get('name') === 'object') {
                    group.set({
                        name        : group.get('name').name,
                        toggled : group.get('name').toggled
                    });
                }
            });

            Session.user.groups.reset(groups.models);

            Roster.groups.sort();

            Roster.limitCustomGroups();
        },

        rosterGroupResetHandler: function () {
            this.log('rosterGroupResetHandler');

            if (_.url.hash('debug-messenger') !== 'true') { return; }

            var jid = Session.user.get('id');
            var groupMapping = {
                1: {
                    toggled : false,
                    name        : gt('Buddylist')
                },
                2: {
                    toggled : false,
                    name        : gt('Favorites')
                },
                3: {
                    toggled : false,
                    name        : gt('Guest Users')
                },
            };

            UCConnect.rosterUserGroupsSet('xmpp', jid, [1,2,3]);
            UCConnect.storePrivateXml('xmpp', 'roster:groups', JSON.stringify(groupMapping));
            UCConnect.getRoster('xmpp');
        },

        /**
         * Handle the creation of a new custom group
         * @param    {Object} object containing the group model and groupName
         * @return {null}
         */
        rosterGroupCreateHandler: function (object) {
            this.log('rosterGroupCreateHandler', object.groupName);

            var newGroup        = Roster.createGroup(object.groupName), // Add the new Group to the Roster
                    groupsObj     = Roster.getGroupsObject(), // Get the group Object, IDs and values
                    users             = Roster.getAllUsers();

            // Save the groupName for this ID in the PrivateXML
            UCConnect.storePrivateXml('xmpp', 'roster:groups', JSON.stringify(groupsObj));

            // Update the groups in the Session User
            Session.user.groups.reset(Roster.getGroups().models);

            if (users) {
                // Add new property for each user, so it's in the dropdown
                users.each(function (user) {
                    user.properties.addProperty(newGroup.get('name'), false);
                });
            }

            Roster.groups.sort();

            Roster.limitCustomGroups();
        },


        /**
         * Handle the update for a new custom group
         * @param    {Object} object containing the group model and new groupName
         * @return {null}
         */
        rosterGroupUpdateHandler: function (object) {
            this.log('rosterGroupUpdateHandler', object);

            var that                 = this,
                    group                = object.group,
                    groupId            = object.group.id,
                    groupName        = object.group.get('name'),
                    newGroupName = object.groupName,
                    newGroup         = Roster.updateGroup(group, newGroupName),
                    groupsObj        = Roster.getGroupsObject(),
                    groupKeys        = Roster.getGroupKeys(),
                    groupValues    = Roster.getGroupValues(),
                    users                = Roster.getGroupUsers(groupId);

            // Save the groupName for this ID in the PrivateXML
            UCConnect.storePrivateXml('xmpp', 'roster:groups', JSON.stringify(groupsObj));

            // Update the groups in the Session User
            Session.user.groups.reset(Roster.getGroups().models);

            Roster.groups.sort();
        },

        // Delete the Group Name
        //
        // object.group = Group Model
        rosterGroupDeletedHandler: function (object) {
            this.log('rosterGroupDeletedHandler', object);

            var that                 = this,
                    group                = object.group,
                    groupId            = object.group.id,
                    groupName        = object.group.get('name'),
                    newGroupName = object.groupName,
                    users                = Roster.getGroupUsers(groupId),
                    deletedGroup = Roster.deleteGroup(groupId),
                    groupsObj        = Roster.getGroupsObject(),
                    groupKeys        = Roster.getGroupKeys(),
                    groupValues    = Roster.getGroupValues();

            // Save the groupName for this ID in the PrivateXML
            UCConnect.storePrivateXml('xmpp', 'roster:groups', JSON.stringify(groupsObj));

            // Update the groups in the Session User
            Session.user.groups.reset(Roster.getGroups().models);

            if (users) {
                // Add new property for each user, so it's in the dropdown
                users.each(function (user) {
                    // that.deleteGroupForUser(user, groupName);
                    user.properties.deleteProperty(groupId);
                    UCConnect.rosterUserGroupsSet('xmpp', user.get('id'), user.properties.toArray());
                });
            }

            Roster.groups.sort();

            Roster.limitCustomGroups();
        },

        // Save the toggle state of the group
        //
        // object.model = RosterGroupModel
        // object.toggled = Boolean
        rosterGroupToggledHandler: function (object) {
            this.log('rosterGroupToggledHandler', object);

            var groupToggleStatuses,
                    group                = Roster.getGroup(object.model.id),
                    currentState = object.model.get('toggled'),
                    newState         = !currentState;

            group.set('toggled', newState);

            groupToggleStatuses = Roster.getGroupsObject();

            UCConnect.storePrivateXml('xmpp', 'roster:groups', JSON.stringify(groupToggleStatuses));
        },

        removeGuestData: function (user) {
            this.log('removeGuestData', user);

            UCConnect.rosterUserRemove('xmpp', user);
            //todo: decide if we also want to remove the chat history
        },

        conversationStartHandler: function (data) {
            this.log('conversationStartHandler', data);

            var id = data.id,
                type = data.type,
                conversation = Session.user.conversations.get(id),
                self = this;

            if (!conversation) {
                conversation = this.startConversation(id, type);
            }

            Roster.hide();


            // Used to block the textInput temporarily
            if(data.delayUserInput) {
                conversation.delayUserInput = true;
            }
            Conversation.showConversation(conversation);

            // TODO: other side isCallable? or both?
            if ((type === 'audio' || type === 'video') && Session.user.isCallable()) {
                var callSession = UCConnect.makeCall(id, type === 'video');

                conversation.callParticipants.reset();

                conversation.set('initiator', Session.user);

                Session.user.call.set({
                    callSession: callSession,
                    type: type,
                    conversationId: id
                });

                // Add call participants
                conversation.addCallParticipant(Session.user, callSession, true, type);

                conversation.getOtherParticipants().each(function (participant) {
                    conversation.addCallParticipant(participant.id, callSession, false, type);
                });

                Session.user.setInCall(true, conversation);
            }
            self.sortConversations(id);

            Aggregator.trigger('conversationlist:selection:change', id);
        },

        conversationStateHandler: function (data) {
            if(Settings.get('showUsersMyChatState')){
                var isGroup = data.conversation instanceof GroupConversationModel;
                var to = data.conversation.id;
                var state = data.state;

                if(!isGroup) {
                    to = to + '/oxmessenger';
                }

                UCConnect.chatState(to, state, isGroup);
            }
        },

        sendMessageHandler: function (message) {
            this.log('sendMessageHandler', message);

            // Send message to UCConnect
            if (message.conversation.isMuc()) {
                UCConnect.groupMessage('xmpp', message.conversation.get('roomHost'), message.conversation.get('roomId'), message.get('message'), message.getFileDescAsObject());
            } else {
                var fullJid = message.conversation.id;
                for (var i in Roster.getUserById(message.conversation.id).get('statuses')) {
                    if (Roster.getUserById(message.conversation.id).get('statuses')[i].status !== 'offline') {
                        fullJid = i;
                        break;
                    }
                }
                UCConnect.message(fullJid, true, message.get('message'), null, message.getFileDescAsObject());
            }
            this.sortConversations();
        },

        groupLeaveHandler: function (data) {
            this.log('groupLeaveHandler', data);

            var conversation = data.conversation;

            conversation.removeParticipant(Session.user);

            conversation.properties.set('groupMember', false);

            return UCConnect.groupLeave('xmpp', conversation.get('roomHost'), conversation.get('roomId'));
        },

        groupJoinHandler: function (data) {
            this.log('groupJoinHandler', data);

            var conversation = data.conversation;
            var avoidMsgFetch = true;

            if (conversation.messageGroups.length) {
                conversation.messageGroups.reset();
            }

            conversation.addParticipant(Session.user, (data.isHistory === undefined) ? false : true);

            conversation.properties.set('groupMember', true);

            UCConnect.groupJoin('xmpp', conversation.get('roomHost'), conversation.get('roomId'),
                    UCConnect.getConnection().jid, avoidMsgFetch);
            UCConnect.fetchMemberList('xmpp', conversation.get('roomHost'), conversation.get('roomId'));
            return;
        },

        /*
         * OX event handlers
         * ================================================
         */

        offlineHandler: function () {
            this.log('offlineHandler connection:offline', event);

            var that = this;

            var timeout = 2000;
            var connect = function() {
                Session.setup();
                // We'll recheck the connection till UCC is properly connected
                setTimeout(function(){
                    // Check if the user is still offline
                    if(Session.get('connection') === 'offline' && !Session.get('connecting')) {
                        connect();
                    }
                }, timeout);
                timeout = Math.min(60000, 2 * timeout);
            };

            // Set UCC as desconnected
            that.disconnect();

            // Once OX is back to online, we try to reconnect
            ox.once('connection:online', function(){
                connect();
            });

        },
        /*
         reloginHandler: function (event) {
         this.log('reloginHandler', event);

         this.disconnect(event);
         ox.once('relogin:success', function () {
         setTimeout(function(){
         Session.setup();
         }, 2000);
         });
         },
         */

        /*
         * UC event handlers
         * ================================================
         */

        callStateHandler: function (event) {
            this.log('callStateHandler', event, event.data.session.display_name, event.data.status);


            if (Session.user.call.isGroup() && event.data.session.display_name === 'send' && event.data.status === 'answered') {
                // Got an 'answered' status from the send application: group call initiated
                var conversation = Session.user.conversations.get(Session.user.call.get('conversationId')),
                        call = event.data.call,
                        type = call.callType,
                        callType = (type === 'video') ? 'groupvideo' : 'groupaudio',
                        self = this;

                Session.user.call.setCallState('group-initiated');

                this.handleSpeakingEvent(conversation, call);

                if (conversation.get('initiator') !== Session.user) {
                    UCConnect.groupCallReady('xmpp', conversation.get('roomHost'), conversation.get('roomId'), type);

                    conversation.callParticipants.each(function (participant) {
                        if (participant.id === Session.user.id) {
                            return;
                        }

                        if (participant.call.get('readyReceived') === false) {
                            return;
                        }

                        // Call the receive application with the id of the participant
                        if (!participant.call.isActive()) {
                            var recvSession = UCConnect.makeCall('recv', (type === 'video'), ['X-HASH: ' + participant.id + ';' + conversation.get('roomId')], self.groupCallEndCallHandler);
                            participant.call.set('callSession', recvSession);
                        }
                    });

                } else {
                    if (Session.user.call.get('rejoin') === true) {
                        UCConnect.groupCallReady('xmpp', conversation.get('roomHost'), conversation.get('roomId'), type);
                        conversation.callParticipants.each(function (participant) {
                            if (participant.id === Session.user.id) {
                                return;
                            }

                            if (participant.call.get('readyReceived') === false) {
                                return;
                            }

                            // Call the receive application with the id of the participant
                            if (!participant.call.isActive()) {
                                var recvSession = UCConnect.makeCall('recv', (type === 'video'), ['X-HASH: ' + participant.id + ';' + conversation.get('roomId')], self.groupCallEndCallHandler);
                                participant.call.set('callSession', recvSession);
                            }
                        });
                    } else {
                        // Create list of invited users
                        var userlist = [];
                        conversation.participants.each(function (participant) {
                            var error = Session.user.canCallWith(type, participant, true);
                            if (participant !== Session.user && error !== true) {
                                return;
                            }

                            //we need to know if is a 1To1 to Muc
                            var invitedUser = null;
                            if(conversation.get('second')){
                                invitedUser = conversation.get('second').id;
                            }

                            userlist.push({
                                user: participant.id,
                                active: Util.isSameJid(participant.id, UCConnect.getConnection().jid),
                                readyReceived: false,
                                invitedUser: invitedUser
                            });
                        });
                        conversation.getOtherParticipants().each(function (participant) {
                            var error = Session.user.canCallWith(type, participant, true);
                            if (error !== true) {
                                return;
                            }
                            UCConnect.groupCallInvite('xmpp', conversation.get('roomHost'), conversation.get('roomId'), participant.get('id'), type, JSON.stringify(userlist));
                        });
                    }
                }
                this.setGroupCallStatus(conversation, true, type);
                return;
            }

            if ((Session.user.call.isGroup() && event.data.session.display_name === 'recv') ||
                !Session.user.call.isGroup()) {
                var conversation = Session.user.conversations.get(Session.user.call.get('conversationId'));

                conversation.callParticipants.each(function (participant) {
                    if (!participant.call.isActive() || participant.call.get('callSession').id !== event.data.session.id) {
                        return;
                    }

                    switch (event.data.status) {
                        case 'trying':
                        case 'in-progress':
                            participant.call.setCallState('in-progress');
                            break;
                        case 'answered':
                            participant.call.setCallState('answered');
                            Util.stopAllSounds();
                            break;
                        case 'ended':
                            participant.call.setCallState('idle');
                            break;
                    }
                });
            }

            Session.user.call.setCallState(event.data.status);
        },

        handleSpeakingEvent: function (conversation, call) {
            this.log('handleSpeakingEvent', conversation, call);

            var listener = function (data) {
                var roomId = conversation.get('roomId');
                var mucHost = Settings.get('groupConversationRoomHost');

                var muted = Session.user.call.get('audioMuted');
                var speaking = Boolean(!muted * data);

                // TODO handle no call-type scenario from #630
                // https://jira.voiceworks.com/browse/OXM-630
                if (!conversation.get('callType'))
                    return;

                UCConnect.groupCallStatus('xmpp', mucHost, roomId, JSON.stringify({
                    // TODO this is a dirty dirty hack
                    callType: conversation.get('callType').replace('group', ''),
                    speaking: speaking
                }));
            };

            call.mediaHandler.on('speaking', listener);
            call.once('bye', call.mediaHandler.off.bind(
                    call.mediaHandler, 'speaking', listener));
        },

        setGroupCallStatus: function (conversation, hasActiveCall, callType) {
            this.log('setGroupCallStatus', conversation, hasActiveCall, callType);

            assert(conversation instanceof ConversationModel, 'Param type mismatch');

            Session.user.set('inGroupCall', conversation.get('roomId'));

            // if (!callType && hasActiveCall) {
            //         callType = conversation.get('callType');
            // }

            var json = {
                roomId: conversation.get('roomId'),
                status: hasActiveCall
            };

            if (callType) {
                assert(CALL_TYPE[callType]);
                // TODO this is a dirty dirty hack
                callType = callType.replace('group', '');
                json.callType = callType;
            }

            UCConnect.groupCallStatus('xmpp', Settings.get('groupConversationRoomHost'), conversation.get('roomId'),
                    JSON.stringify(json));
        },

        connectingHandler: function (event) {
            this.log('connectingHandler', event);

            this.setSessionServerState(event);
            if (event.proto !== 'xmpp') {
                return;
            }

            Session.user.setStatus('offline');
        },

        connectedHandler: function (event) {
            this.log('connectedHandler', event);

            this.setSessionServerState(event);
            if (event.proto !== 'xmpp') {
                    if (location.protocol === 'https:') {
                            Session.set('server_sip_try_counter', 0);
                            Aggregator.trigger('sipSession:connected');
                            return;
                    }
                    else {
                            CoreNotification.yell('warning', gt('Audio and video call capabilities are disabled. Please log into the client with HTTPS to enable them again.'));
                            setTimeout( function () {
                                    Aggregator.trigger('sipSession:disconnected');
                                },500);
                            return;
                    }
            }

            // Remove all conversations
            Session.user.conversations.reset();

            if (Settings.get('initialPresenceStatus')) {
                Session.user.setStatus(Settings.get('initialPresenceStatus'));
            }

            Session.set('connection', 'online');

            CallHistory.fetch(Settings.get('callHistoryLimit'));

            Session.user.set({
                canChat: UCConnect.capabilities.chat,
                canAudio: UCConnect.capabilities.audio,
                canVideo: UCConnect.capabilities.video,
                mustFlash: UCConnect.capabilities.mustFlash
            });

            Aggregator.trigger('conversation:reloadCurent');
        },

        disconnectedHandler: function (event) {
            this.log('disconnectedHandler', event);

            if (Session.user.call) {
                //Session.user.call.get('callSession').terminate();
                Session.user.call.end();
            }

            this.setSessionServerState(event);

            if (event.proto === 'sip' &&
                !Session.get('connecting') &&
                Session.get('connection') === 'online' &&
                Session.get('server_sip') === 'disconnected' &&
                Session.get('server_xmpp') === 'connected'
            ) {
                if (typeof Session.get('server_sip_try_counter') === 'undefined' ) {
                    Session.set('server_sip_try_counter', 0);
                }

                if (Session.get('server_sip_try_counter') > 0) {
                    Aggregator.trigger('sipSession:disconnected');
                }
                Session.set('server_sip_try_counter', Session.get('server_sip_try_counter') + 1);
                return;
            }
            else if (event.proto === 'sip' &&
                !Session.get('connecting') &&
                Session.get('connection') === 'offline' &&
                Session.get('server_sip') === 'disconnected' &&
                Session.get('server_xmpp') === 'connecting'
            ) {
                Aggregator.trigger('sipSession:disconnected');
                return;
            }
            else if (event.proto === 'sip' &&
                !Session.get('connecting') &&
                Session.get('connection') === 'offline' &&
                Session.get('server_sip') === 'disconnected' &&
                Session.get('server_xmpp') === 'disconnected'
            ) {
                Aggregator.trigger('sipSession:disconnected');
                return;
            }

            // If both XMPP and SIP were disconnected, our session's been probably stolen


            Session.user.setStatus('offline');
            Session.set('connection', 'offline'); // normally a user won't go offline

            this.disconnect();
        },

        endCallHandler: function (event) {
            this.log('endCallHandler', event);

            var conversationId = Session.user.call.get('conversationId') ? Session.user.call.get('conversationId') : Session.user.call.previous('conversationId'),
                    conversation = Session.user.conversations.get(conversationId),
                    pstnRegExp = new RegExp('@' + Settings.get('pstnCallDomain') + '$', 'i'),
                    session = event.data.session,
                    isPstn = session && event.data.session.sip_uri.match(pstnRegExp),
                    cause = event.data.cause ? event.data.cause : '',
                    callType = event.data.call ? event.data.call.callType : '';

            if (!conversation && event.data.call && event.data.call.request.extraHeaders) {
                conversationId = Util.findRoomIdFromHeaders(event.data.call.request.extraHeaders);
                conversation = Session.user.conversations.get(conversationId + '@' + Settings.get('groupConversationRoomHost'));
            }

            if (Session.user.call.isGroup()) {
                var participantsToRemove = [];

                if (session && session.display_name === 'send') {
                    Session.user.call.end();
                    Session.user.setInCall(false);

                    conversation.callParticipants.each(function (participant) {
                        participant.call.end();
                    });

                    conversation.callParticipants.reset();

                    this.setGroupCallStatus(conversation, false);

                    return;
                }

                if (session && session.display_name === 'recv') {
                    var participant = conversation.callParticipants.find(function (p) {
                        return p.call.get('callSession') && (p.call.get('callSession').id === event.data.session.id);
                    });

                    if (participant) {
                        participant.call.end();
                        conversation.callParticipants.remove(participant);

                        if (event.data.cause!=='NORMAL_CLEARING') {
                            Aggregator.trigger('muc:tryCallAgain', conversation, participant);
                        }
                    }
                }

                // Hang up if you are the last one in the group call
                if (conversation.callParticipants.length < 2) {
                    Session.user.call.end();
                    Session.user.setInCall(false);

                    conversation.callParticipants.reset();
                    this.setGroupCallStatus(conversation, false);
                } else {
                    this.setGroupCallStatus(conversation, true, conversation.get('callType'));
                }

                return;
            }

            // Hangup 1:1 calls
            Session.user.setInCall(false);
            Session.user.endCall();
            if(conversation) {
                conversation.callParticipants.each(function (participant) {
                    participant.call.end();
                });
                conversation.callParticipants.reset();
            }

            var isFailed = false;

            if (typeof event.data.call !== 'undefined') {

                isFailed        = event.data.session.status === 'failed';
                var localHost     = event.data.call.localIdentity.uri.host,
                        localUser     = event.data.call.localIdentity.uri.user,
                        remoteHost    = event.data.call.remoteIdentity.uri.host,
                        remoteUser    = event.data.call.remoteIdentity.uri.user,
                        isInbound     = event.data.call.direction === 'inbound',
                        historyItem = new HistoryModel({
                            billmsec: (isFailed) ? '0' : event.data.call.endTime.getTime() - event.data.call.startTime.getTime(),
                            call_direction: isInbound ? 'inbound' : 'outbound',
                            effective_destination_domain: isInbound ? localHost : remoteHost,
                            effective_destination_user: isInbound ? localUser : remoteUser,
                            effective_from_domain: isInbound ? remoteHost : localHost,
                            effective_from_user: isInbound ? remoteUser : localUser,
                            hangup_cause: (isFailed) ? 'CALL_REJECTED' : 'NORMAL_CLEARING',
                            start_stamp: (isFailed) ? new Date().toISOString() : new Date(event.data.call.startTime).toISOString()
                        });

                // Add call item to Call History and conversation
                Session.user.callHistory.add(historyItem);

                // Do not add Call History items for PSTN calls and group calls
                //if (!isPstn && event.data.session.display_name !== 'send' && event.data.session.display_name !== 'recv') {
                if (!isPstn && event.data.session.display_name !== 'send') {
                    if (!conversation) {
                        conversation = this.startConversation(historyItem.getOtherParty(), 'chat');
                        this.sortConversations(null);
                    }

                    conversation.addMessage(conversation.createCallHistoryItem(historyItem));
                }

            }

            if (conversation && conversation.isMuc()) {
                var frozenParticipants = conversation.participants.filter(function (p) {
                    return p.get('lastCallFrozen') === true;
                });

                if (frozenParticipants.length > 0) {
                    CoreNotification.yell('warning', gt('The call was interrupted due to network problems. Please verify that you have a stable internet connection.'));
                }

                // conversation.unset('lastCallFrozen');
            } else if (conversation && cause !== 'NORMAL_CLEARING' && event.data.cause!=='rejected') {

                if ( Session.get('server_xmpp') === 'connected' && Session.get('server_sip') === 'connected') {
                        Aggregator.trigger('call:issueDetected', gt('Sorry, the connection failed due to a network error.'), conversation.get('id'), callType);
                }
                else {
                        CoreNotification.yell('warning', gt('The call was interrupted due to network problems. Please verify that you have a stable internet connection.'));
                        conversation.unset('lastCallFrozen');
                }
            } else if (conversation && conversation.get('lastCallFrozen')) {

                CoreNotification.yell('warning', gt('The call was interrupted due to network problems. Please verify that you have a stable internet connection.'));
                conversation.unset('lastCallFrozen');

            }

            //@TODO add event handler for failing PSTN
            if (isFailed) {
                Util.playSound('busy.mp3', 'audioBusy', false);
            }

            if ($('#videoTmp').length) {
                $('#videoTmp').remove();
            }

            if ($('.messenger--notifier-container[data-uri="' + event.data.session.sip_uri + '"]').length) {
                Notification.call.destroy();
            }
        },

        handleTryCallAgain: function (conversation, participant) {

            this.log('handleTryCallAgain', conversation, participant);
            CoreNotification.yell('warning', gt('Sorry, the connection to some of the participants failed. Please end the call and use the "Join Active Call" button to solve the error.'));
        },

        failedHandler: function (event) {
            this.log('failedHandler', event);

            CoreNotification.yell('error', gt('There was a problem connecting Messenger. Please, try again later.'));
        },

        localMediaRequestedHandler: function (event) {
            this.log('localMediaRequestedHandler', event);
        },

        localMediaFailedHandler: function (event) {
            this.log('localMediaFailedHandler', event);
        },

        localMediaAcquiredHandler: function (event) {
            this.log('localMediaAcquiredHandler', event);

            if (Util.isFirefox()) {
                Aggregator.trigger('call:startHangUpTimeout', Session.user.conversations.currentConversationId);
            }
        },

        rsmHandler: function (event) {
            var roomJid = event.data.from;
            var first = event.data.first;
            var last = event.data.last;
            var count = event.data.count;
            var conversation = Session.user.conversations.get(roomJid);
            var lastLocalStorage = conversation.getLocalStorage() || _conversationsStorage[roomJid] || 0;

            conversation.set('totalCount', count);

            // If the conversation is still empty (no messages loaded yet / not initialized)
            if(!conversation.get('lastRsmId')) {
                this.log('conversationlocalStorage',
                    conversation.get('roomName') + ' -> count: ' + count +
                    ' ,lastStorage: ' + lastLocalStorage);

                if(Session.user.conversations.currentConversationId === conversation.id) {
                     conversation.setUnreadMessages(0);
                }

                if(lastLocalStorage || lastLocalStorage === 0) {
                    conversation.setUnreadMessages(Math.max(count - lastLocalStorage, 0));
                } else if(!lastLocalStorage ||
                    lastLocalStorage < 0) {
                    conversation.updateLocalStorage(count);
                }

                this.sortConversations(null, true);
            }

            // Where first & last are message IDs in the DB
            conversation.setRsmData(first, last, count);
        },

        messageHandler: function (event) {
            this.log('messageHandler()', event, event.data);

            var message = MessageFactory.create(event.data),
                self = this,
                conversation;

            if (message.getSenderJid() == UCConnect.getConnection().jid &&
                !message.get('isHistory') && !message.get('isCarbonCopy')) {
                return;
            }

            if (MessageFactory.isSystem(message)) {
                conversation = Session.user.conversations.get(message.getRoomJid());
                // system messages are silent and modify stuff like room details
                // for now we only deal with room name change coming in with 'subject'
                if (!message.has('subject') || !message.get('subject').length) {}
                else {
                        //conversation.set('subject', message.get('subject'));
                }
                // Temporarily ignore system messages
                return;
            } else if (MessageFactory.isGroup(message)) {
                conversation = Session.user.conversations.get(message.getRoomJid());
            } else { // chat
                if (message.get('isCarbonCopy')) {
                        conversation = this.startConversation(message.get('carbonTo'), 'chat');
                } else {
                    conversation = this.startConversation(message.getSenderJid(), 'chat');
                }
            }

            var msgDisplayName = '',
                msgSenderJid = '',
                msgImageUrl = '',
                user = null;

            if ((typeof message.user) !== 'undefined') {
                user                 = message.user;
                msgSenderJid = message.getSenderJid();
            }
            else {
                msgSenderJid = Util.getBareJid(event.data.nick);
                user = conversation.participants.get(msgSenderJid);
            }

            if (user) {
                msgDisplayName = user.getDisplayName();
                msgImageUrl = user.getImageUrl();
            }

            if ((typeof message.user) === 'undefined') {
                var msgArray = message.get('message').split(' '),
                    participantJid = Util.getUserIdFromJid(msgArray.shift()),
                    messageText = msgArray.join(' ');

                message.set('message', gt('%s %s', Roster.getUserById(participantJid).getDisplayName(), messageText));
            }

            // Show desktop and call notifications based on settings
            NotificationControl.notify(conversation, {
                desktop: function () {
                    if (message.get('isHistory') || message.get('message') === '') {
                        return;
                    }

                    self.desktopNotification.show({
                        title: msgDisplayName,
                        message: message.get('message'),
                        tag: msgSenderJid,
                        icon: msgImageUrl,
                        callback: function () {
                            Aggregator.trigger('conversation:show', { id: (MessageFactory.isGroup(message)) ? message.getRoomJid() : message.getSenderJid(), type: 'chat' });
                            try
                            {
                                Aggregator.trigger('focus:textarea');
                            }
                            catch (ex) { }
                        }
                    });
                }
            });

            conversation.addMessage(message);

            // Don't sort conversations if messenger is not visible
            if (this._currentAppName === 'com.voiceworks/ox-messenger') {
                this.sortConversations();
            }
        },

        /**
         * TODO
         *
         * This nasty debounce is a temp hack and should be placed in the
         * rendering layer.
         *
         * Selection logic should be handled separately from sorting.
         */
        sortConversations: _.debounce(function(id, ignoreOldSelection) {
            this.log('sortConversations', id);

            if (Conversation.listView) {
                var oldSelection = Conversation.listView.selection.get();

                Session.user.conversations.sort();
                Session.user.conversations.trigger('reset');

                // Select the conversation with the provided ID
                if(id) {
                    var conversation = Session.user.conversations.get(id);

                    Conversation.listView.selection.set([conversation.cid]);
                }
                // If there's no ID provided, and there were conversations selected, keep them selected
                else if(!ignoreOldSelection && oldSelection.length > 0){
                    Conversation.listView.selection.set(oldSelection);
                }
                // If there's no ID and there was no conversation selected, select the first one by default
                else if (Session.user.conversations.length > 0){
                    var firstConv = Session.user.conversations.first();

                    Session.user.conversations.currentConversationId = null;
                    Conversation.listView.selection.set([firstConv.cid]);
                }
            }
        }, 100),

        myStatusHandler: function (event) {
            this.log('myStatusHandler', event);
            var resourceId = Util.getResourceFromJid(event.data.username);

            // sync status changes from remote resources (but skip the initial status)
            var localResource = Util.getResourceFromJid(UCConnect.getConnection().jid);
            if (parseInt(event.data.status.priority, 10) > 1 &&
                    (event.data.status.status !== Session.user.get('status')) &&
                    resourceId != localResource) {
                Session.user.setStatus(event.data.status.status);
                UCConnect.status(event.data.status.status, event.data.status.message);
            }

            // Update user data based on vcard
            if (event.data.vcard) {
                var avatarUrl = event.data.vcard.avatarUrl;

                if (avatarUrl) {
                    Session.user.setImageUrl(avatarUrl);
                    UserPart.updateAvatars(Session.user);
                }
            }

            UserPart.updatePresenceBadges(Session.user.id, event.data.status.status);
        },

        newCallHandler: function (event) {
            this.log('newCallHandler', event);

            var sip_uri = event.data.session.sip_uri,
                    user = Roster.getUserById(sip_uri),
                    userDisplayName = user.getDisplayName(),
                    self = this,
                    type = event.data.call.callType, //request.body.match('m=video') !== null ? 'video' : 'audio',
                    typetxt = '',
                    conversation = Session.user.conversations.get(user.get('jid'));

            assert(CALL_TYPE[type], 'Call type incorrect');

            switch (type) {
                case 'video':
                    typetxt = gt('%s is video calling you...', userDisplayName);
                    break;
                case 'audio':
                    typetxt = gt('%s is calling you...', userDisplayName);
                    break;
                case 'groupvideo':
                    typetxt = gt('%s is group video calling you...', userDisplayName);
                    break;
                case 'groupaudio':
                    typetxt = gt('%s is group calling you...', userDisplayName);
                    break;
            }

            // Show desktop and call notifications based on settings
            NotificationControl.notify(conversation, {
                desktop: function () {
                    self.desktopNotification.show({
                        title: userDisplayName,
                        message: typetxt,
                        tag: sip_uri,
                        icon: user.getImageUrl(),
                        callback: function () {
                            try {
                                window.focus();
                            } catch (ex) { }
                        }
                    });
                },
                call: function () {
                    Notification.call.create(user, type, event.data.session, conversation);
                }
            });

            // @TODO when to ignore this?
            // @TODO rewrite from field in UCConnect Library to show full from instead of only user part of remote identity
        },

        callAcceptHandler: function (event) {
            this.log('callAcceptHandler', event);

            // Dont start a new call if one is already ongoing.
            if (!Session.user.isCallable()) {
                return;
            }

            var conversationId = event.session.sip_uri,
                conversationType = event.session.call.callType, //request.body.match('m=video') !== null ? 'video' : 'audio',
                conversation = this.startConversation(conversationId, conversationType),
                user = event.user;

            assert(CALL_TYPE[conversationType]);

            conversation.callParticipants.reset();

            conversation.set('initiator', Session.user);
            
            conversation.addCallParticipant(Session.user, event.session, true, conversationType);
            conversation.addCallParticipant(user.id, event.session, true, conversationType);

            Session.user.call.set({
                callSession: event.session,
                type: conversationType,
                conversationId: conversationId
            });

            event.session.answerCall(conversationType === 'video');

            Session.user.setInCall(true, conversation);

            this.sortConversations(conversation.id);

            Conversation.showConversation(conversation);
        },

        callDeclineHandler: function (event) {
            this.log('callDeclineHandler', event);

            try {
                event.session.terminateCall();

            } catch (e) {
                console.error('Call termination error', e);
            }

            //todo: add system message in conversation to log declined call;
        },

        callUnsuccessfulHandler: function (data) {
            this.log('callUnsuccessfulHandler', data);

            var conversation = Session.user.getCurrentCallConversation();
            if (conversation) {
                var participant = conversation.callParticipants.get(data);

                if (participant) {
                    conversation.callParticipants.remove(participant);
                }

                // Hang up if you are the last one in the group call
                if (conversation.callParticipants.length < 2) {
                    Session.user.call.end();
                    Session.user.setInCall(false);

                    conversation.callParticipants.reset();
                    this.setGroupCallStatus(conversation, false);
                } else {
                    this.setGroupCallStatus(conversation, true, conversation.get('callType'));
                }

            }

        },

        callIssueDetectedHandler: function (issueMessage, conversationId, callType) {

            this.log('callIssueDetectedHandler', issueMessage);

            require([
                'io.ox/core/tk/dialogs'
            ], function (dialogs) {
                new dialogs.ModalDialog()
                        .text(issueMessage)
                        .addPrimaryButton('tryToReccnnect', gt('Try to reconnect'), 'tryToReconnect', { tabIndex: '1' })
                        .addButton('close', gt('Close'), 'close', { tabIndex: '1' })
                        .show()
                        .done(function (action) {
                            if (action === 'tryToReccnnect') {
                                Aggregator.trigger('call:init', { id: conversationId, type: callType });
                            } else if(action === 'close') {
                                Aggregator.trigger('focus:textarea');
                            }
                        });
            });
        },

        callHangupHandler: function (data) {
            
            this.log('callHangupHandler', data);

            if (!Session.user.call.isActive()) {
                return;
            }

            var otherUserCnt = 0;

            // Hangup all participants
            data.conversation.callParticipants.each(function (participant) {
                if (participant === Session.user) {
                    return;
                }

                if (participant.call.isActive()) {
                    participant.call.end();
                    otherUserCnt++;
                }
            });
            data.conversation.callParticipants.reset();
            data.conversation.groupCallInvitees.reset();


            if (Session.user.call.isGroup()) {
                // Cancel group call if you are the originator and you are not connected to other participants yet
                if (data.conversation.get('initiator') === Session.user && otherUserCnt === 0) {
                    UCConnect.groupCallCancel('xmpp', data.conversation.get('roomHost'), data.conversation.get('roomId'), Session.user.call.get('type'));
                }
                this.setGroupCallStatus(data.conversation, false);
            }

            Session.user.call.end();
            var convesationId = data.conversation.isMuc()?data.conversation.get('id'):Session.user.get('id');
            Session.user.setInCall(false, data.conversation, 'event:hangup:'+convesationId);
        },

        parametersHandler: function (event) {
            this.log('parametersHandler', event);
        },

        mucsHandler: function (event) {
            _.each(event.data.mucs, function (room) {
                var conversation = new GroupConversationModel({
                    id: room.jid,
                    roomId: Util.getBareJid(room.jid, true),
                    roomName: room.name,
                    roomHost: Settings.get('groupConversationRoomHost'),
                    title: room.name,
                    myRoom: true,
                    subject: room.subject,
                    user: Session.user,
                    sortName: room.name,
                    presenceReceived: false,
                    timestamp: -1
                });
                Session.user.conversations.add(conversation);

                // Join conversation + fetch member list
                Aggregator.trigger('group:join', {
                    conversation: conversation,
                    isHistory: true
                });
            });
            Session.user.conversations.sort();
            Session.user.conversations.trigger('reset');
            Aggregator.trigger('muchistory:loaded');
        },

        rosterHandler: function (event) {
            this.log('rosterHandler', event);
            var rosterObj = event.data.roster;

            // Fetch the roster groups (after receibing the roster)
            UCConnect.fetchPrivateXml('xmpp', 'roster:groups', function (data) {
                var storedGroups,
                    groups = Roster.getGroups();

                // Check if the returned data is a valid JSON object
                try{
                    storedGroups = JSON.parse(data);
                }catch(e){
                    Roster.create(rosterObj);
                    return;
                }

                // Find missing groups between PrivateXML and Roster
                var rosterGroupIds = Roster.getGroups().models.map(function (group) { return parseInt(group.id, 10); }),
                    storedGroupIds = _.keys(storedGroups).map(function (group) { return parseInt(group, 10); }),
                    missingGroups    = _.difference(storedGroupIds, rosterGroupIds);

                // Add missing groups
                for(var groupId in storedGroups) {
                    if(!groups.get(parseInt(groupId))){
                        Roster.createGroup(storedGroups[groupId], parseInt(groupId, 10));
                    }
                }

                // Set names from PrivateXML for all groups
                groups.models.forEach(function(group) {
                    if (typeof group.get('name') === 'object') {
                        group.set({
                            name: group.get('name').name,
                            toggled: group.get('name').toggled
                        });
                    }
                });

                Session.user.groups.reset(groups.models);

                Roster.groups.sort();

                // Finally, create the roster
                Roster.create(rosterObj);
            });
        },

        setSessionServerState: function (event) {
            this.log('setSessionServerState', event);

            var label = 'server_' + event.proto;
            Session.set(label, event.type);

            if (event.proto !== 'xmpp') {
                return;
            }

            if (event.type === 'disconnected') {
                Aggregator.trigger('session:disconnected', event);
            }
        },

        statusHandler: function (event) {
            this.log('statusHandler', event);

            this.statusExtraEventsHandler(event);

            if (event.data.muc) {
                this.groupStatusHandler(event);
            } else {
                Roster.setStatus(event.data);
            }
        },

        statusExtraEventsHandler: function(event) {
            this.log('statusExtraEventsHandler', event);

            var statuses = event.data.status;

            for (var jid in statuses) {
                var status =    statuses[jid];
                if ((typeof status.message !== 'undefined') &&
                    (status.message !== '')) {
                    var messageExt = status.message.split(':');
                    if (messageExt[0] === 'event') {
                        var eventType = messageExt[1];

                        switch (eventType) {
                            case 'hangup':
                                var conversationId = messageExt[2];
                                var conversation = Session.user.conversations.get(conversationId);
                                if (conversation) {
                                    var user = Roster.getUserById(jid);
                                    var sysMessage = new SystemMessageModel({
                                            message: gt('%s ended the conversation', user.getDisplayName()),
                                            timestamp: new Date().getTime()
                                    });
                                    conversation.addMessage(sysMessage);
                                }
                                break;
                        default:
                            this.log('statusExtraEventsHandler unknown event', event);
                            break;
                        }
                    }
                }
            }
        },

        vcardUpdatedHandler: function (event) {
            this.log('vcardUpdatedHandler', event);
            var user = Roster.getUserById(event.data.username),
                    firstName = event.data.vcard.firstName,
                    lastName = event.data.vcard.lastName,
                    avatarUrl = event.data.vcard.avatarUrl;

            if (firstName && lastName) {
                user.setFullName(firstName, lastName);
                UserPart.updateNickname(user);
            }

            if (avatarUrl) {
                user.setImageUrl(avatarUrl);
                UserPart.updateAvatars(user);
            }

            var conversation = Session.user.conversations.get(event.data.username);

            if (conversation) {
                conversation.set('title', lastName + ', ' + firstName);
            }

        },

        chatStateHandler: function (event) {
            var conversationJid = Util.getBareJid(event.data.from);
            var isGroup = event.data.isGroup;
            var state = event.data.state;
            var theConversation;
            var fromJid;
            var fromUser;

            // Find the conversation
            Session.user.conversations.each(function (conversation) {
                if(conversation.id === conversationJid) {
                    theConversation = conversation;
                }
            });

            // If the conversation exist
            if(theConversation){
                if(isGroup){
                    fromJid = Util.getResourceFromJid(event.data.from, true);
                }
                else{
                    fromJid = conversationJid;
                }


                // Find the participant
                theConversation.participants.each(function (participant) {
                    if(participant.id === fromJid){
                        fromUser = participant;
                    }
                });

                // If the participant exists (and is NOT me), update conversation composing participants
                if(fromUser && fromUser.id !== Session.user.id){
                    if(state === ConversationModel.STATE_COMPOSING){
                        theConversation.addComposingParticipant(fromUser);
                    }
                    else{
                        theConversation.removeComposingParticipant(fromUser);
                    }
                }
            }
        },

        subscribeHandler: function (event) {
            this.log('subscribeHandler', event);

            if (event.data.username !== '') {
                var pendingInvitations = [].concat(Settings.get('pendingInvitations'));
                pendingInvitations.push({ jid: event.data.username });
                Settings.set('pendingInvitations', pendingInvitations);
                Settings.save();
                Aggregator.trigger('buddylist:reset-invitations');
            }
        },

        groupCreateHandler: function (data) {
            this.log('groupCreateHandler', data);

            var that = this,
                isModerated = false,
                conversation = data.conversation,
                succesfullyAdded = 0;

            if (!conversation) {
                CoreNotification.yell('error', gt('Group could not be created'));
            }

            // Create and configure the group
            var success = UCConnect.groupCreate(
                'xmpp',
                conversation.get('roomHost').toLowerCase(),
                conversation.get('roomId'),
                conversation.get('roomName'),
                conversation.get('subject'),
                UCConnect.getConnection().jid,
                isModerated,
                function createParticipants() {
                    _.each(data.participants, function (participant) {
                        // Do not invite yourself
                        if (Util.isSameJid(participant.id, UCConnect.getConnection().jid)) {
                                return;
                        }

                        var isInvited = that.groupInviteHandler({
                                conversation: conversation,
                                user: participant,
                                notify: false
                        });

                        if (isInvited) {
                                succesfullyAdded++;
                        }
                    });

                    if (succesfullyAdded === 0 && data.participants.length > 0) {
                        CoreNotification.yell('error', gt('Group could not be created'));
                    }

                    conversation.getNextRsm(1);

                    Aggregator.trigger('groupconversation:changed', {});
                }
            );

            if (!success) {
                CoreNotification.yell('error', gt('Group could not be created'));
            }
        },

          groupCreateNewHandler: function (data) {
              this.log('groupCreateHandler', data);

              var that = this,
                isModerated = false,
                conversation = data.conversation,
                succesfullyAdded = 0,
                invitedUser = data.invitedUser,
                callType;

              if (!conversation) {
                  CoreNotification.yell('error', gt('Group could not be created'));
              }

              // Create and configure the group
              var success = UCConnect.groupCreate(
                'xmpp',
                conversation.get('roomHost').toLowerCase(),
                conversation.get('roomId'),
                conversation.get('roomName'),
                conversation.get('subject'),
                UCConnect.getConnection().jid,
                isModerated,
                function createParticipants() {
                    _.each(data.participants, function (participant) {

                        callType = (participant.call.get('type') === 'video') ? 'groupvideo' : 'groupaudio';
                        participant.call.end();
                        Session.user.call.end();
                        Session.user.setInCall(false);

                        var isInvited = that.groupInviteHandler({
                            conversation: conversation,
                            user: participant,
                            notify: false
                        });

                        if (isInvited) {
                            succesfullyAdded++;
                        }
                    });

                    if (succesfullyAdded === 0 && data.participants.length > 0) {
                        CoreNotification.yell('error', gt('Group could not be created'));
                    }

                    conversation.getNextRsm(1);

                    Aggregator.trigger('conversation:show', {
                        id: conversation.id,
                        type: (callType === 'groupvideo') ? 'groupvideo' : 'groupaudio'
                    });

                    Aggregator.trigger('groupconversation:changed', {});

                    //we need this a bit later or xmpp stuff does not work, what we do here is killing old call and creating a muc
                    setTimeout( function () {
                        Aggregator.trigger('group:createNewCall', { conversation: conversation, type: (callType === 'groupvideo') ? 'video' : 'audio', invitedUser: invitedUser });
                    },1500);

                }
              );

              if (!success) {
                  CoreNotification.yell('error', gt('Group could not be created'));
              }
          },

        groupCreateNewCallHandler : function(data) {
            this.log('groupCreateNewCallHandler', data);

            Aggregator.trigger('groupcall:init', { conversation: data.conversation, type: data.type, invitedUser: data.invitedUser });

        },

        groupInviteHandler: function (data) {
            this.log('groupInviteHandler', data);

            var conversation = data.conversation,
                user = data.user,
                notify = data.notify,
                success = false,
                callType = Session.user.call.get('type');

            success = UCConnect.groupInvite('xmpp', conversation.get('roomHost').toLowerCase(),
                      conversation.get('roomId'), conversation.get('roomName'), user.id);

            if (success) {
                if (notify) {
                    CoreNotification.yell('info', gt('%s has been added to the conversation', user.getDisplayName()));

                    var joinMessage = user.id === conversation.user.id ? gt('You joined the group conversation') : gt('%s joined the group conversation', user.getDisplayName()),
                        sysMessage = MessageFactory.create({
                            message     : joinMessage,
                            timestamp : new Date().getTime(),
                            muc             : true,
                            from            : conversation.get('roomId') + '@' + conversation.get('roomHost').toLowerCase()
                        });

                    conversation.addMessage(sysMessage);

                    return;
                }

                return true;
            } else {
                CoreNotification.yell('error', gt('Error adding %s', user.getDisplayName()));
            }

            return false;
        },

        groupInvitedHandler: function (event) {
            this.log('groupInvitedHandler', event);

            var data = event.data,
                    roomJid = data.roomId + '@' + data.host;

            if (Session.user.conversations.get(roomJid) && Session.user.conversations.get(roomJid).participants.get(Util.getBareJid(UCConnect.getConnection().jid))) {
                return;
            }

            var conversation = new GroupConversationModel({
                id: roomJid,
                roomId: data.roomId,
                roomName: data.roomName,
                roomHost: data.host,
                title: data.roomName,
                myRoom: false,
                user: Session.user,
                timestamp: new Date().getTime(),
                sortName: data.roomName
            });
            conversation.updateLocalStorage(0);

            Session.user.conversations.add(conversation);

            UCConnect.groupJoin('xmpp', event.data.host, event.data.roomId, UCConnect.getConnection().jid, true);
            UCConnect.fetchMemberList('xmpp', event.data.host, event.data.roomId);
            this.sortConversations();
        },

        groupStatusHandler: function (event) {
            this.log('groupStatusHandler', event.data.username, event);

            var conversation = Session.user.conversations.get(event.data.roomId + '@' + event.data.host),
                    user = Roster.getUserById(event.data.username);

            // Get the first Muc Page after joining (when we receive our own presence on a conversation)
            if (Util.isSameJid(event.data.username, UCConnect.getConnection().jid) && conversation) {
                conversation.getNextRsm(1);
            }

            var currentRoomId = Session.user.get('inGroupCall');
            if (!conversation || Util.isSameJid(event.data.username, UCConnect.getConnection().jid) ||
                    !currentRoomId || currentRoomId != conversation.get('roomId')) {
                return;
            }

            conversation.sendGroupCallInvite(user, false);

            var groupCallStatus = Boolean(conversation.callParticipants &&
                    conversation.callParticipants.get(Util.getBareJid(UCConnect.getConnection().jid)));
            var callType = '';

            if (user.call) {
                callType = user.call.get('type');
                if (!callType && conversation.hasActiveCall()) {
                    callType = conversation.get('callType');
                }
            }

            this.setGroupCallStatus(conversation, groupCallStatus, callType);
        },

        groupSubjectHandler: function (data) {
            this.log('groupSubjectHandler', data);
            var conversation = data.conversation;

            UCConnect.groupSubject(
                    'xmpp', conversation.get('roomHost').toLowerCase(),
                    conversation.get('roomId'),
                    conversation.get('subject')
            );
        },

        /*
         * PSTN handlers
         * ================================================
         */

        phoneCallHandler: function (data) {
            this.log('phoneCallHandler', data);

            // Dont start a new call if one is already ongoing.
            if (Session.user.callActive) {
                return;
            }

            var session = UCConnect.makeCall('sip:' + data.number + '@' + Settings.get('pstnCallDomain'), false);
            Session.user.call.set({
                callSession: session,
                type: 'pstn'
            });

            Session.user.setInCall(true);
        },

        phoneHangupHandler: function (data) {
            this.log('phoneHangupHandler', data);

            Session.user.call.end();
            Session.user.setInCall(false);
        },

        phoneHoldHandler: function (data) {
            this.log('phoneHoldHandler', data);

        },

        phoneDtmfHandler: function (data) {
            this.log('phoneDtmfHandler', data);

            switch (data.key) {
                case '0':
                case '1':
                case '2':
                case '3':
                case '4':
                case '5':
                case '6':
                case '7':
                case '8':
                case '9':
                    Util.playSound('dtmf/' + data.key + '.mp3', 'dtmf', false);
                    break;
                case '*':
                    Util.playSound('dtmf/asterisk.mp3', 'dtmf', false);
                    break;
                case '#':
                    Util.playSound('dtmf/pound.mp3', 'dtmf', false);
                    break;
            }
            return Session.user.call.isActive() && Session.user.call.get('callSession').sendDTMF(data.key);
        },

        /*
         * Groupcall handlers
         * ================================================
         */

        // This is a method which is required for bypassing the Chrome bug in group calls. It counts the active number of participants to let JsSIP know if the RTCPeerConnection should be closed
        // or that it should put it in some sort of sleep mode until all calls are ended.
        groupCallEndCallHandler: function (data) {
            //this.log('groupCallEndCallHandler', data);

            var xHeader = null,
                    conversationId = null,
                    conversation = null;

            if (data.remoteIdentity.uri.user === 'send') {
                return true;
            }

            for (var i in data.request.extraHeaders) {
                if (data.request.extraHeaders[i].indexOf('X-HASH') > -1) {
                    xHeader = data.request.extraHeaders[i];
                    break;
                }
            }
            if (xHeader) {
                xHeader = xHeader.split(';');
                conversationId = xHeader[1];
                conversation = Session.user.conversations.get(conversationId + '@' + Settings.get('groupConversationRoomHost'));
                if (!conversation) {
                    return true; // no conversation found, must be something wrong but the call has to be ended
                }
                if (conversation.callParticipants.length - 1 < 2) {
                    return true; // this was the final receiver session, end call
                } else {
                    return false; // there are more receiver sessions active, do not terminate the call
                }
            } else {
                return true; // no conversation found, must be something wrong but the call has to be ended
            }
            return false;
        },

        groupCallStartHandler: function (data) {
            this.log('groupCallStartHandler', data);

            // Dont start a new call if one is already ongoing.
            if (!Session.user.isCallable()) {
                return;
            }

            var callType = data.type === 'video' ? 'groupvideo' : 'groupaudio',
                    warnings = [];

            var invitedUser = data.invitedUser;

            var sendSession = null;

            sendSession = UCConnect.makeCall('send', data.type === 'video',
              ['X-HASH: ' + Util.getBareJid(Session.user.id) + ';' + data.conversation.get('roomId')],
              this.groupCallEndCallHandler);

            data.conversation.callParticipants.reset();

            // Add yourself to the list of (active) call participants
            data.conversation.addCallParticipant(Session.user, sendSession, true, callType);

            data.conversation.set('initiator', Session.user);

            if (invitedUser) {
                data.conversation.set('second', invitedUser);
            } else {
                data.conversation.set('second', null);
            }

            Session.user.call.set({
                callSession: sendSession,
                type: callType,
                conversationId: data.conversation.id
            });

            data.conversation.getOtherParticipants().each(function (participant) {
                var error = Session.user.canCallWith(data.type, participant, true);
                if (error !== true) {
                    warnings.push(error);
                    return;
                }

                data.conversation.addCallParticipant(participant.id, null, false, callType);

            });

            if (warnings.length) {
                CoreNotification.yell('warning', warnings.join('\n\n'));
            }

            Session.user.setInCall(true, data.conversation);

            if (invitedUser) {
                Aggregator.trigger('group:invite', {
                    conversation: data.conversation,
                    user: invitedUser,
                    notify: true
                });
                //this will trigger adding the third invitted to the new muc from 1To1
                setTimeout( function () {
                    data.conversation.groupCallInvitees.add(invitedUser);
                    data.conversation.sendGroupCallInvite(invitedUser, true);
                },1500);
            }

            // Set a timeout in case all participants ignore the call
            data.conversation.startActivityTimeout(function (activity) {
                if (!activity){
                    Aggregator.trigger('call:hangup', {conversation: data.conversation});
                }
            });
        },

        groupCallSendInviteHandler: function (data) {
            this.log('groupCallSendInviteHandler', data);

            var userlist = [],
                    user         = data.user,
                    callType     = Session.user.call.get('type').replace('group', ''),
                    invitedUser  = data.invitedUser;

            var canCallWith = Session.user.canCallWith(callType, user, true);
            if (canCallWith !== true) {
                CoreNotification.yell('info', gt('%s has been added to the conversation, but cannot join the call. He has been asked to join using Chrome or Firefox.', user.getDisplayName()));
                return false;
            }

            if(invitedUser){
                invitedUser = data.user.get('id');
            } else {
                invitedUser = null;
            }

            data.conversation.callParticipants.each(function (participant) {
                userlist.push({
                    user: participant.id,
                    active: Util.isSameJid(participant.id, UCConnect.getConnection().jid),
                    readyReceived: participant.call.get('readyReceived'),
                    invitedUser : invitedUser
                });
            });

            assert(CALL_TYPE[callType]);
            UCConnect.groupCallInvite('xmpp', data.conversation.get('roomHost'), data.conversation.get('roomId'), data.user.get('id'), callType, JSON.stringify(userlist));
        },

        groupCallInviteHandler: function (event) {
            this.log('groupCallInviteHandler', event);

            // Dont accepts a new call if one is already ongoing.
            if (!Session.user.isCallable()) {
                return;
            }

            Session.user.callRinging = true;

            assert(CALL_TYPE[event.data.type]);

            var callType = event.data.type === 'video' ? 'groupvideo' : 'groupaudio',
              initiator = Roster.getUserById(event.data.username),
              conversation = Session.user.conversations.get(event.data.roomId + '@' + event.data.host),
              isOneToMany = false;

            if (!conversation) {
                return;
            }

            conversation.callParticipants.reset();

            _.each(event.data.participantlist, function (participant) {
                //there is an invitedUser from 1-1- to 1-N and this session user is not that one, do not show the invite beacuse this user was in the 1-1
                if (participant.invitedUser && (participant.user !== participant.invitedUser) && participant.user === Session.user.id) {
                    isOneToMany = true;
                }

                if (Util.isSameJid(participant.user, UCConnect.getConnection().jid)) {
                    return;
                }

                // Add user to participants list (if necessary)
                if (!conversation.participants.get(participant.user)) {
                    conversation.participants.add(Roster.getUserById(participant.user));
                }

                // Add user to call participants list (if necessary)
                if (!conversation.callParticipants.get(participant.user)) {
                    conversation.addCallParticipant(participant.user, null, participant.user === event.data.username || participant.readyReceived, callType);
                }
            });

            conversation.set('initiator', initiator);

            var tmpSession = {
                sip_uri: conversation.get('id')
            };

            var sip_uri = conversation.get('id'),
              user = Roster.getUserById(sip_uri),
              userDisplayName = initiator.getDisplayName(),
              typetxt = '';

            switch (callType) {
                case 'groupvideo':
                    typetxt = gt('%s is group video calling you...', userDisplayName);
                    break;
                case 'groupaudio':
                    typetxt = gt('%s is group calling you...', userDisplayName);
                    break;
            }

            // Dispose the state on accept or reject, unbind the listener as well.
            var dropRingingState = function () {
                Session.user.callRinging = false;
                Aggregator.off('groupcall:accept', dropRingingState);
                Aggregator.off('groupcall:decline', dropRingingState);
                UCConnect.events.removeListener('groupcallcancelled', dropRingingState);
            };
            Aggregator.once('groupcall:accept', dropRingingState);
            Aggregator.once('groupcall:decline', dropRingingState);
            // TODO should be once, but doesnt work
            UCConnect.events.addListener('groupcallcancelled', dropRingingState);

            //if is a muc from 1To1 and I am not the 3rd invitee, meaning I was in the call already, except invitation, don't show it
            if (isOneToMany) {
                Aggregator.trigger('groupcall:accept', {
                    conversation: conversation,
                    type: event.data.type,
                    caller: initiator
                });
            } else {
                this.desktopNotification.show({
                    title: userDisplayName,
                    message: typetxt,
                    tag: sip_uri,
                    icon: user.getImageUrl(),
                    callback: function () {
                        try {
                            window.focus();
                        } catch (ex) {
                        }
                    }
                });

                Notification.call.create(initiator, callType, tmpSession, conversation);
            }

            this.setGroupCallStatus(conversation, true, callType);
        },

        groupCallReadyHandler: function (event) {
            this.log('groupCallReadyHandler', event);

            assert(CALL_TYPE[event.data.type]);

            var callType = event.data.type === 'video' ? 'groupvideo' : 'groupaudio',
                    conversation = Session.user.conversations.get(event.data.roomId + '@' + event.data.host),
                    participant = conversation.callParticipants.get(Util.getBareJid(event.data.from));

            // Ignore self
            if (Util.isSameJid(event.data.from, UCConnect.getConnection().jid)) {
                return;
            }

            if (!participant) {
                var jid = Util.getBareJid(event.data.from);
                conversation.addCallParticipant(jid, null, true, callType);
                participant = conversation.callParticipants.get(jid);
            }

            participant.call.set('readyReceived', true);

            // if you're not a callParticipant, you haven't accepted the invitation (yet)
            if (conversation.callParticipants.get(Util.getBareJid(UCConnect.getConnection().jid))) {
                Util.stopAllSounds();
                var recvSession = UCConnect.makeCall('recv', event.data.type === 'video', ['X-HASH: ' + Util.getBareJid(event.data.from) + ';' + event.data.roomId], this.groupCallEndCallHandler);
                participant.call.set('callSession', recvSession);
            }

            this.setGroupCallStatus(conversation, Session.user.call.isActive(), callType);
        },

        groupCallCancelledHandler: function (event) {
            this.log('groupCallCancelledHandler', event);

            var conversation = Session.user.conversations.get(event.data.username);
            Util.stopAllSounds();

            conversation.callParticipants.each(function (participant) {
                participant.call.end();
            });

            Session.user.call.end();
            Session.user.setInCall(false);

            // Remove notification
            if ($('.messenger--notifier-container[data-uri="' + conversation.id + '"]').length) {
                Notification.call.destroy();
            }
            this.setGroupCallStatus(conversation, false);
        },

        groupCallAcceptHandler: function (call) {
            this.log('groupCallAcceptHandler', call);

            assert(CALL_TYPE[call.type]);

            var isVideo = call.type === 'video',
                    callType = isVideo ? 'groupvideo' : 'groupaudio',
                    self = this;

            var sendSession = UCConnect.makeCall('send', isVideo,
                    ['X-HASH: ' + Util.getBareJid(UCConnect.getConnection().jid) + ';' + call.conversation.get('roomId')],
                    this.groupCallEndCallHandler);

            Session.user.call.set({
                callSession: sendSession,
                type: callType,
                conversationId: call.conversation.id
            });

            Session.user.setInCall(true, call.conversation);

            call.conversation.addCallParticipant(Session.user, sendSession, true, callType);

            // When Rejoining a Call
            if (call.rejoin) {
                Session.user.call.set('rejoin', true);
                this.groupCallRejoin(call, callType);
            }

            Aggregator.trigger('conversation:show', {
                id: call.conversation.id,
                type: isVideo ? 'groupvideo' : 'groupaudio'
            });

            this.setGroupCallStatus(call.conversation, true, callType);
        },

        groupCallRejoin: function (call, callType) {
            this.log('groupCallRejoin', call, callType);

            assert(CALL_TYPE[callType]);

            var callParticipants = call.conversation.getCallParticipants();

            _.each(callParticipants, function (participant) {
                if (participant === Session.user) {
                    return;
                }
                call.conversation.addCallParticipant(participant.get('jid'), null, true, callType);
                participant.call.set('readyReceived', true);
            });

            if (call.conversation.get('initiator') === Session.user) {
                //UCConnect.groupCallReady('xmpp', call.conversation.get('roomHost'), call.conversation.get('roomId'), call.type);
            }
        },

        groupCallDeclineHandler: function (data) {
            this.log('groupCallDeclineHandler', data);

            assert(CALL_TYPE[data.type]);

            var result = UCConnect.groupCallDecline('xmpp', data.conversation.get('roomHost'), data.conversation.get('roomId'), data.type);
            this.setGroupCallStatus(data.conversation, false);
        },

        groupCallDeclinedHandler: function (event) {
            this.log('groupCallDeclinedHandler', event);

            var conversation = Session.user.conversations.get(event.data.roomId + '@' + event.data.host),
                    participant = conversation.callParticipants.get(Util.getBareJid(event.data.from));

            if (!conversation) {
                return;
            }

            if (participant) {
                conversation.callParticipants.remove(participant);
            }

            this.setGroupCallStatus(conversation, conversation.getCallParticipantsCount() > 1,
                    conversation.get('callType'));

            // End call if you are the only one left
            if (conversation.getCallParticipantsCount() === 1){
                Aggregator.trigger('call:hangup', {conversation: conversation});
            }
        },

        groupCallStatusHandler: function (event) {
            this.log('groupCallStatusHandler', event);

            var searchJid = event.data.roomId + '@' + event.data.host,
                    conversation = Session.user.conversations.get(searchJid);

            if (!conversation) {
                return;
            }

            var participant = conversation.participants.get(Util.getBareJid(event.data.from));
            if (!participant) {
                return;
            }

            var json = JSON.parse(Strophe.xmlunescape(event.data.status));

            if (json.speaking !== undefined) {
                if (Session.user.call.get('callState') === 'idle' && participant.get('isSpeaking') === true) {
                    participant.set('isSpeaking', false);
                } else {
                    participant.set('isSpeaking', !!json.speaking);
                }
            }

            if (json.status !== undefined && (!participant.get('inGroupCall') ||
                    participant.get('inGroupCall') == json.roomId)) {
                var participantsInGroupCall;

                participant.set('inGroupCall', json.status ? json.roomId : '');
                participantsInGroupCall = conversation.getNumParticipantsInGroupCall();

                // Hangup the call if I'm the only one in the group and people are hanging up (n_ready > n_participants)
                // This is a hack for firefox to close the group calls
                // when the other participants hangup and there's no focus in firefox
                // or the permissions window is not accepted
                if (Session.user.id !== participant.id && // Make sure the event is sent by another user
                        Session.user.call.get('callState') === 'group-initiated' &&
                        participantsInGroupCall === 1 &&
                        conversation.getNumParticipantsReady() > participantsInGroupCall) {
                    this.log('groupCall:hangup', 'Only one participant remaining');
                    Aggregator.trigger('call:hangup', {conversation: conversation});
                }
                else if (Session.user.id !== participant.id && // Make sure the event is sent by another user
                        Session.user.call.get('callState') === 'idle' &&
                        participantsInGroupCall === 0) {
                        Notification.call.decline();
                }

            }

            if (json.callType) {
                assert(CALL_TYPE[json.callType]);
                conversation.set('callType', json.callType);
            }
        },

        /*
         * Helpers
         * ================================================
         */
        inviteUser: function (jid) {
            this.log('inviteUser', jid);

            if (!UCConnect.rosterUserAdd('xmpp', jid)) {
                return false;
            }
            if (!UCConnect.rosterUserSubscribe('xmpp', jid)) {
                return false;
            }
            return true;
        },

        authorizeUser: function (jid) {
            this.log('authorizeUser', jid);

            return UCConnect.rosterUserAuthorize('xmpp', jid);
        },

        /**
         *
         * @param email {string}
         * @returns {Promise}
         */
        addContact: function (email) {
            this.log('addContact', email);

            // Adding that buddy as contact in your address book, but be avare that untill he accept the invitation,
            // he will not be on lists for adding in conversations.
            return ContactsApi.create({
                folder_id: CoreSettings.get('folder/contacts'),
                email1: email
            }).done(function (contact) {
                Aggregator.trigger('session:reloadContacts');
            }).fail(function (e) {
                console.error(e);
            });
        },

        unauthorizeUser: function (jid) {
            this.log('unauthorizeUser', jid);

            return UCConnect.rosterUserUnauthorize('xmpp', jid);
        },

        disconnect: function (event) {
            this.log('disconnect', event);

            Session.set('connection', 'offline');
            UCConnect.SessionHandler.terminateAll();

            Aggregator.trigger('session:disconnected', event);

            UCConnect.disconnect();
        },

        isOnline: function () {
            this.log('isOnline');

            return Session.get('connection') === 'online';
        },

        startConversation: function (id, type) {
            this.log('startConversation', id, type);

            var participant = Roster.getUserById(id);
            return Conversation.start(participant, type);
        },

        handleSettingsChatNotifications: function (event) {
            this.log('handleSettingsChatNotifications', event);

        },

        handleConversationNotifications: function (event) {
            this.log('handleConversationNotifications', event);

            var savedSettings = Settings.get('conversationNotificationSettings'),
                    newSettings = [],
                    conversation = event.conversation,
                    conversationJid = conversation.get('id');

            // Remove all matches for this jid
            var newSettings = _.reject(savedSettings, function (s) {
                return s.jid === conversationJid;
            });

            // Save setting
            newSettings.push({
                'jid': conversationJid,
                'showChatNotifications': event.showChatNotifications
            });

            Settings.set('conversationNotificationSettings', newSettings);
            Settings.save();
        },

        setupConversationNotificationSettings: function (event) {
            this.log('setupConversationNotificationSettings', event);

            if (!event) {
                return;
            }

            var conversationId = event.get('id'),
                    conversation = Session.user.conversations.get(conversationId);

            if (!conversation) {
                return;
            }

            var currentSetting = _.findWhere(Settings.get('conversationNotificationSettings'), {jid: conversationId});

            if (currentSetting) {
                conversation.set('showNotifications', currentSetting.showChatNotifications);
                conversation.properties.set('showNotifications', currentSetting.showChatNotifications);
            }
        },

        onAppChange: function (app) {
            // Check if it's a real app change
            if (this._currentAppName !== app.get('name')) {
                // If switch to a different app (not Messenger), keep streams alive
                if ((app.get('name') !== 'com.voiceworks/ox-messenger')){}
                // If we resume Messenger, refresh the current conversation
                else if (app.get('state') === 'running') {
                    var currentConversation =
                            Session.user.conversations.get(Session.user.conversations.currentConversationId),
                        forceCreate = true;

                    // If active call is the focused one, show it
                    Conversation.showConversation(currentConversation, forceCreate);

                    // Added to sort conversation after showing it again
                    this.sortConversations();
                }

                this._currentAppName = app.get('name');
            }
        },

        handleReloadCurentConversation: function() {
            var currentConversation = Session.user.conversations.get(Session.user.conversations.currentConversationId);

            Conversation.showConversation(currentConversation);
        },

        log: function () {
            if (_.url.hash('debug-messenger') && window.console) {
                console.info('mediator: ' + _.first(arguments), _.rest(arguments));
            }
        }
    };

    return Mediator;
});
