/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/roster/views/new_group', [
    'io.ox/core/extensions',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/roster/models/group',
    'gettext!com.voiceworks/ox-messenger'
], function (ext, Aggregator, RosterGroupModel, gt) {
    'use strict';

    /**
     * Roster Search View
     */
    var RosterNewGroupView = Backbone.View.extend({
        className: 'messenger--edit-group-title',

        initialize: function (options) {
            // defaults
            this.model         = this.model || new RosterGroupModel();


            //this.toggleElement = $('<a href="#" class="messenger--edit-group-title__toggle">');
            this.toggleElement = $('<button class="btn btn-primary messenger--edit-group-title__toggle control">');
            this.toggleLabel   = this.toggleElement.text(gt('Add new group')).prepend($('<i class="fa fa-plus">'));
            this.showToggle    = false;
            this.inputValue    = '';
            this.submitType    = 'create';

            // custom options
            if (options && options.toggleElement) {
                this.toggleElement = options.toggleElement.addClass('messenger--edit-group-title__toggle');
            }
            if (options && options.toggleLabel) {
                this.toggleElement.text(options.toggleLabel);
            }
            if (options && options.showToggle) {
                this.showToggle = options.showToggle;
            }
            if (options && options.inputValue) {
                this.inputValue = options.inputValue;
            }
            if (options && options.submitType) {
                this.submitType = options.submitType;
            }

            this.render();
        },

        events: {
            'click .messenger--edit-group-title__toggle'       : 'handleToggle',
            'click .messenger--edit-group-title__form__accept' : 'handleSubmit',
            'click .messenger--edit-group-title__form__cancel' : 'handleCancel',
            'submit .messenger--edit-group-title__form'        : 'handleSubmit',
        },

        render: function () {
            this.$el.children().remove();
            this.$el.prevObject = null;

            this.inputField = $('<input class="form-control search-field messenger--edit-group-title__form__input">').attr({
                type: 'text',
                'placeholder': gt('Add a new group ...')
            });
            this.buttonAccept = $('<a href="#" class="fa fa-check  messenger--edit-group-title__form__accept">').attr({
                'title': gt('Save')
            });
            this.buttonCancel = $('<a href="#" class="fa fa-close  messenger--edit-group-title__form__cancel">').attr({
                'title': gt('Cancel / Delete')
            });

            this.$el.append(
                $('<form class="messenger--edit-group-title__form  messenger--is-hidden">').append(
                    this.inputField,
                    this.buttonAccept,
                    this.buttonCancel
                ),
                $('<div class="messenger--edit-group-title__validation messenger--h5 messenger--is-hidden">').text(gt('Group name is required'))
            );

            if (this.showToggle) {
                this.$el.prepend(this.toggleElement);
                this.toggleElement.removeClass('messenger--is-hidden');
            }

            if (this.inputValue.length > 0) {
                this.inputField.val(this.inputValue);
            }
            var error = gt('Adding a new group is not possible. You already have the maximum of ten groups.');
            this.$el.find('.btn').attr('data-original-title', error);
            this.$el.find('.btn').attr('data-placement', 'bottom');

            // Enable/disable
            if(this.$el.find('.btn').hasClass('disabled'))
              this.$el.find('.btn').tooltip('disable');
            else
              this.$el.find('.btn').tooltip('enable');

            return this;
        },

        handleCancel: function (event) {
            event.preventDefault();
            event.stopPropagation();

            Aggregator.trigger('roster:group:edit:cancel', event);

            this.render();
        },

        handleToggle: function (event) {
            event.preventDefault();
            event.stopPropagation();

            if (this.$el.find('.btn').hasClass('disabled')) {
                return;
            }
            this.toggle();
        },

        handleSubmit: function (event) {
            event.preventDefault();
            event.stopPropagation();

            this.toggle();

            var inputField = this.$el.find('.messenger--edit-group-title__form__input').first(),
                value      = _.escape(inputField.val());

            $(event.target).val('');

            var submitted = this.submit(value);

            if (submitted) {
                this.render();
            } else {
                this.toggle();
            }
        },

        toggle: function () {
            if (this.showToggle) {
                this.$el.find('.messenger--edit-group-title__toggle').toggleClass('messenger--is-hidden');
            }

            this.$el.find('.messenger--edit-group-title__form').toggleClass('messenger--is-hidden');

            this.$el.find('.messenger--edit-group-title__form__input').focus();
        },

        show: function (show) {
            if (show) {
                this.$el.find('.btn').removeClass('disabled');
                this.$el.find('.btn').tooltip('disable');
            }
            else {
                this.$el.find('.btn').addClass('disabled');
                this.$el.find('.btn').tooltip('enable');
            }
        },

        submit: function (value) {
            if (value.length === 0) {
                this.$el.find('.messenger--edit-group-title__validation').removeClass('messenger--is-hidden');
                this.$el.find('.messenger--edit-group-title__form__input').addClass('has-error');
                return false;
            }

            var object = {
                group: this.model,
                groupName: value
            };

            if (this.submitType === 'create') {
                Aggregator.trigger('roster:group:create', object);
            } else if (this.submitType === 'update') {
                Aggregator.trigger('roster:group:update', object);
            }

            return true;
        },
    });

    return RosterNewGroupView;
});
