/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/roster/views/user', [
    'io.ox/core/extensions',
    'com.voiceworks/ox-messenger/util',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/core/views/disposable',
    'com.voiceworks/ox-messenger/core/views/dropdown',
    'com.voiceworks/ox-messenger/core/user',
    'gettext!com.voiceworks/ox-messenger',
    'settings!com.voiceworks/ox-messenger',
], function (ext, Util, Aggregator, DisposableView, Dropdown, UserModel, gt, settings) {
    'use strict';

    /**
     * Roster User View
     *
     * this.collection = RosterUserCollection
     */
    var RosterUserView = DisposableView.extend({
        tagName: 'li',

        initialize: function (options) {
            DisposableView.prototype.initialize.apply(this, arguments);

            this.user = options.user;
            this.model = this.model || new UserModel();

            this.listenTo(this.model, 'change', this.render);
            this.listenTo(this.user.properties, 'change', this.render);
            this.listenTo(Aggregator, 'dropdown:item', this.handleDropdownItemClicked);
        },

        events: {
            'click .messenger-js--start-chat': 'startChat',
            'click .messenger-js--start-audio': 'startAudio',
            'click .messenger-js--start-video': 'startVideo',
            'click .messenger-js--remove-guest': 'removeGuest',
            'click .messenger-js--change-group': 'renderDropdown',
        },

        startChat: function (e) {
            if (this.model.get('subscription') === 'both') {
                Aggregator.trigger('conversation:show', { id: this.model.id, type: 'chat' });
            }
            e.stopPropagation();
        },

        startAudio: function (e) {
            if (this.model.get('subscription') === 'both') {
                Aggregator.trigger('conversation:show', { id: this.model.id, type: 'audio' });
            }
            e.stopPropagation();
        },

        startVideo: function (e) {
            if (this.model.get('subscription') === 'both') {
                Aggregator.trigger('conversation:show', { id: this.model.id, type: 'video' });
            }
            e.stopPropagation();
        },

        removeGuest: function (e) {
            Aggregator.trigger('roster:remove-guest', { user: this.model });
            e.stopPropagation();
        },

        render: function () {
            var avatar = this.model.getImageUrl('large'),
                favorite,
                remove;

            this.$el.children().remove();
            this.$el.empty();
            this.$el.prevObject = null;

            var chat = (this.user.canChatWith(this.model) === true && this.model.isSubscribed()) ?
                $('<button class="messenger--buddy__actions__action  is-status--online  messenger-js--start-chat">').attr({
                    tabindex: '3',
                    type: 'button'
                }).append(
                    $('<i class="fa fa-comment">')
                    ) : '';

            var audio = (this.user.canAudioWith(this.model) === true && this.model.isSubscribed()) ?
                $('<button class="messenger--buddy__actions__action  is-status--online  messenger-js--start-audio">').attr({
                    tabindex: '3',
                    type: 'button'
                }).append(
                    $('<i class="fa fa-phone">')
                    ) : '';

            var video = (this.user.canVideoWith(this.model) === true && this.model.isSubscribed()) ?
                $('<button class="messenger--buddy__actions__action  is-status--online  messenger-js--start-video">').attr({
                    tabindex: '3',
                    type: 'button'
                }).append(
                    $('<i class="fa fa-video-camera">')
                    ) : '';

            if (this.model.isFavorited()) {
                favorite = $('<button class="messenger--buddy__actions__action  is-status--online  messenger-js--unfavorite">').attr({ tabindex: '3', type: 'button' }).append(
                    $('<i class="fa fa-star-o">')
                    );
            } else {
                favorite = $('<button class="messenger--buddy__actions__action  is-status--online  messenger-js--favorite">').attr({ tabindex: '3', type: 'button' }).append(
                    $('<i class="fa fa-star">')
                    );
            }

            var group = $('<div class="messenger--buddy__actions__action  messenger--buddy__actions__action--groups  messenger-js--change-group">').attr({ tabindex: '3', type: 'button' }).append(
                $('<i class="fa fa-users">')
            );

            if (this.model.isGuest()) {
                group = '';
                remove = $('<button class="messenger--buddy__actions__action  is-status--dnd  messenger-js--remove-guest">').attr({ tabindex: '3', type: 'button' }).append(
                    $('<i class="fa fa-remove">')
                    );
            } else {
                remove = '';
            }


            this.$el.append(
                $('<div class="messenger--media  messenger--media--buddy messenger-js--start-chat">').addClass('is-status--' + this.model.get('status')).append(
                    $('<div class="messenger--buddy__actions  is-status-available">').append(
                        chat,
                        audio,
                        video,
                        group,
                        remove
                        ),
                    $('<a class="messenger--media__img  media__img-avatar  messenger--avatar  messenger--avatar--small">')
                        .attr({jid: this.model.id, href: '#', style: 'background-image: url(' + avatar + ')'}),
                    $('<div class="messenger--media__body">').attr({jid: this.model.id}).append(
                        $('<div class="messenger--h5  messenger--font--bold  messenger--buddy__username  messenger--text-wrap">').text(
                            this.model.getDisplayName()
                            ).prepend(
                            $('<span class="messenger--presence">').addClass('is-status--' + this.model.get('status'))
                            ),
                        $('<div class="messenger--h5  messenger--buddy__status">').text(
                            this.model.getDisplayStatus()
                            )
                        )
                    )
                );

            return this;
        },

        renderDropdown: function (event) {
            event.preventDefault();
            event.stopPropagation();

            var groupToggle = $(event.currentTarget),
                buddy = this.model,
                dropdown = groupToggle.siblings('.dropdown');

            var dropdownView = new Dropdown({
                model: this.model.properties,
                extraModels: {
                    user: this.model
                },
                className: 'dropdown open',
                toggleLabel: '',
                toggleIcon: 'users'
            }).header(gt('Add buddy to group:'));


        // Get all the groups without the defaults: buddylist (1), favorites (2) and guest_users (3)
            var customGroups = this.user.groups.reject(function(group) {
                return [1,2,3].indexOf(parseInt(group.id, 10)) !== -1;
            });

            // Add an extra divider between the dropdown header and the favorites group
            if (customGroups.length > 0) {
                dropdownView.divider();
            }

            // Create an option for each custom group
            _.each(customGroups, function (group) {
                var name  = group.id,
                    label = group.get('name');

                dropdownView.option({
                    name: name,
                    label: label
                });
            });

            dropdownView.option({
                name: 2,
                label: gt('Favorites')
            });

            dropdownView.divider();
            dropdownView.option({
                name: 'delete',
                custom: {
                    trueState: {
                        label: gt('Delete buddy'),
                        icon: 'trash-o'
                    },
                    falseState: {
                        label: gt('Delete buddy'),
                        icon: 'trash-o'
                    }
                }
            });

            groupToggle.empty().append(dropdownView.$el);
            dropdownView.render().$el;
        },

        handleDropdownItemClicked: function (data) {

            var user = this.model;

            if (!user || !data || !data.extraModels) {
                return;
            }

            var otherUser = data.extraModels.user;

            if (!otherUser) {
                return;
            }

            if (user.get('id') !== otherUser.get('id')) {
                return;
            }

            if (data.name === 'delete') {
                Aggregator.trigger('roster:remove-buddy', {
                    user: user,
                    values: user.properties.toArray()
                });
            }
            else {
                Aggregator.trigger('roster:group:changed', {
                    user: user,
                    values: user.properties.toArray()
                });
            }
        }

    });

    return RosterUserView;
});
