var plan = require('flightplan');
var os = require('os')
var path = require('path')

// Settings variables
var config = {
    username: 'arturoc',
    remoteBaseFolder: 'messenger/',
    remoteDefaultFolder: 'build',
    remoteDestFolder1: '/opt/open-xchange/appsuite/',
    remoteDestFolder2: '/var/www/appsuite/',
    remoteExec: '/opt/open-xchange/sbin/touch-appsuite'
};

var baseConfig = {
    username: config.username,  // Ask server admin to deploy your RSA public key
    agent: process.env.SSH_AUTH_SOCK
};

Object.assign(baseConfig, os.type() == 'Linux'
    ? { privateKey: path.join(os.homedir(), '.ssh', 'id_rsa') }
    : { agent: process.env.SSH_AUTH_SOCK }
)

// Environment(s) configuration
plan.target('dev', Object.assign({}, baseConfig, {
    host: 'sc-dev3.voiceworks.nl'
}));

plan.target('test', Object.assign({}, baseConfig, {
    host: 'vw68.voiceworks.nl'
}));

// Run commands on localhost
plan.local(function(local) {
    var filesToCopy;

    // Build the project
    local.log('Build');
    local.exec('grunt build', {silent: true});

    // Define the files to be transfered to remoteBaseFolder
    local.log('Copy files to default remote folder: ' + config.remoteBaseFolder);
    filesToCopy = local.exec('find build -type f', {silent: true});
    local.transfer(filesToCopy, config.remoteBaseFolder);
});

// Deploys a new 'build'
plan.remote(function(remote) {
    remote.log('Move files to: ' + config.remoteDestFolder1);

    // Deploy the files from the default folder
    deploy(remote, config.remoteDefaultFolder);

    remote.log('Execute OX touch-appsuite');
    // Copy files from default folder to remote1
    remote.exec(config.remoteExec, {user: config.username});
});


// Stablishes the last 'build' as a new release
plan.remote('save', function(remote) {
    var lastReleaseFolder = getLastReleaseFolder(remote),
        newReleaseFolder = lastReleaseFolder + 1;

    if(lastReleaseFolder < 0)
        throw new Error('There is no build yet');

    remote.log('Saved last build as: ', newReleaseFolder);
    // Rename 'build' folder as a new release
    remote.exec('mv ' +
        config.remoteBaseFolder + config.remoteDefaultFolder + ' ' +
        config.remoteBaseFolder + newReleaseFolder,
        {user: config.username}
    );
});

// Rollsback to last stable release
plan.remote('rollback', function(remote) {
    var lastReleaseFolder = getLastReleaseFolder(remote);

    if(lastReleaseFolder <= 0)
        throw new Error('There is no release to rollback to');

    remote.log('Rollback to release: ', lastReleaseFolder);

    // Deploy the files from the last release folder
    deploy(remote, lastReleaseFolder);

    remote.log('Execute OX touch-appsuite');
    // Copy files from default folder to remote1
    remote.exec(config.remoteExec, {user: config.username});

    // Delete default build folder
    remote.exec('rm -rf ' +
        config.remoteBaseFolder + config.remoteDefaultFolder);
});

// -------------------------------------------- Helpers -----------------------------
function deploy(remote, srcFolder){
    // Copy files from default folder to remote1
    remote.exec('cp -r ' +
        config.remoteBaseFolder + srcFolder + '/apps/ ' +
        config.remoteDestFolder1,
        {user: config.username});

    remote.log('Move files to: ' + config.remoteDestFolder2);
    // Copy files from default folder to remote1
    remote.exec('cp -r ' +
        config.remoteBaseFolder + srcFolder + '/apps/ ' +
        config.remoteDestFolder2,
        {user: config.username});
}

function getLastReleaseFolder(remote){
    var remoteFolders = getRemoteFolders(remote),
        lastRelease;

    // There's no build yet
    if(remoteFolders.length === 0)
        return -1;

    // Filter out the default folder
    remoteFolders = remoteFolders.filter(function(release){
        return release !== config.remoteDefaultFolder;
    });

    // Consider no Release as 0
    if(remoteFolders.length === 0){
        lastRelease = 0;
    }else{
        remoteFolders = remoteFolders.sort(function(a, b){return b - a;});
        lastRelease = parseInt(remoteFolders[0], 10);
    }

    return lastRelease;
}

// Returns an array of remote folders in the Base Folder
function getRemoteFolders(remote){
    var remoteFolders = remote.exec('ls ' + config.remoteBaseFolder, {silent: true});

    if (remoteFolders.code === 0) {
        if (!remoteFolders.stdout)
            throw new Error('There was a problem getting the releases');;

        return remoteFolders.stdout.trim().split('\n');
    }

    return [];
}
