define([
    'io.ox/core/extensions',
    'com.voiceworks/ox-messenger/conversation/models/conversation',
    'com.voiceworks/ox-messenger/conversation/models/message',
    'com.voiceworks/ox-messenger/conversation/models/messagecollection',
    'com.voiceworks/ox-messenger/conversation/models/messagecontainercollection',
    'com.voiceworks/ox-messenger/conversation/models/messagecontainer',
    'com.voiceworks/ox-messenger/core/user',
    'com.voiceworks/ox-messenger/core/sessionuser',
    'com.voiceworks/ox-messenger/roster/models/usercollection',
], function (ext, ConversationModel, MessageModel, MessagesCollection, MessageContainerCollection, MessageContainer, RosterUserModel, SessionUser, RosterUserCollection) {

    describe('ConversationModel ', function () {
        var conversation,
            timestamp,
            user;


        before(function () {
            this.clock = sinon.useFakeTimers();
            timestamp = new Date().getTime();
        });

        after(function () {
            this.clock.restore();
        });

        beforeEach(function () {
            user = new SessionUser();
            conversation = new ConversationModel({user: user});
        });

        it('should have a participants collection', function () {
            expect(conversation.participants).to.be.instanceOf(RosterUserCollection);
        });

        it('should have a messages container collection', function () {
            expect(conversation.messageGroups).to.be.instanceOf(MessageContainerCollection);
        });

        it('should set the correct values with addMessage', function () {
            var message = new MessageModel({message: 'test message'});
            conversation.addMessage(message);
            expect(conversation.get('lastMsg').get('message')).to.equal('test message');
            expect(conversation.get('timestamp')).to.equal(timestamp);
            expect(conversation.messages.pop()).to.equal(message);
        });

        it('should set the correct values with addParticipants', function () {
            var participant = new RosterUserModel({jid: 1, user:user});
            conversation.addParticipant(participant);
            expect(conversation.participants.pop()).to.equal(participant);
        });
    });

});
